#!/usr/bin/env python3

# SPDX-Licence-Identifier: GPL-3.0-or-later

import glob
import os
import sys

from obj_view.window import Window
from obj_view.parse import base_path, parse_obj

ROOT = str(base_path() / 'artwork/gfx/ship/3d')

window = Window(0)

stdout = sys.stdout
sys.stdout = open(os.devnull, 'w')

for path in glob.glob(f'{ROOT}/*/*/*.obj'):
   try:
      ship = parse_obj(path)
   except Exception as e:
      stdout.write(f"Error loading '{path}': {e!r}\n")



#!/usr/bin/python
import os
import sys
import polib
import subprocess
import tempfile
import language_tool_python

"""
This scripts extracts all the gettext translatable fields from a file and then
runs languagetool on each of the outputs. The output follows the same format as
languagetool while keeping the line numbers and offsets relative to the
original input file.

Below is a chunk from an example vimrc of getting this to work with vim
```
" Set up ALE plugin used plugged
call plug#begin('~/.vim/plugged')
   Plug 'dense-analysis/ale'
call plug#end()

" Create the Lua linter
call ale#handlers#languagetool#DefineLinter('lua')
" Set the tool path to the custom tool, should be wherever you have it
let g:ale_languagetool_executable = "/PATH/TO/naev/utils/gettextlint.py"

" Enable the linter by adding it to the defaults
let g:ale_linters = {
\\  'lua':['languagetool', 'cspell', 'lua_language_server', 'luac', 'luacheck', 'selene'],
\\}
```
"""
from pathlib import Path

assert(len(sys.argv)>1)

basepath = Path(__file__).parent

os.environ["LC_ALL"] = "C"

# TODO better handling parameters and ideally passing them on to xgettext
filename = sys.argv[1]
if filename=='--autoDetect':
   filename = sys.argv[2]

with open( filename, 'r' ) as f:
   lines = f.read().splitlines()

# There is no option but to output to a temporary file which we can then pass to languagetool
basename, ext = os.path.splitext(filename)
tf = tempfile.NamedTemporaryFile( suffix='.po' )
if ext==".xml":
   args = [ 'itstool', '--its', basepath / '../po/its/translation.its', '-o', tf.name, filename ] # itstool does not add .po
else:
   args = [ "xgettext", filename, '--from-code=utf-8', '-d', tf.name[:-3] ] # xgettext adds .po again
ret = subprocess.run( args )

# Load the dictionary
with (basepath / "languagetool_dict.txt").open() as f:
   lt_dict = f.read().splitlines()
lt_dict = list( filter( lambda x: x[0]!='#', lt_dict ) )

# Run languagetool with custom dictionary and Oxford british on the generated po text
tool = language_tool_python.LanguageTool( language='en-GB', newSpellings=lt_dict)
tool.enabled_categories = ['BRE_STYLE_OXFORD_SPELLING']
po = polib.pofile( tf.name )
n = 1
for entry in po:
   for ti,txt in enumerate(entry.msgid.splitlines()):
      line = int(entry.occurrences[0][1])+ti
      f = lines[line-1].find(txt)
      if f < 0:
         continue
      assert( f>= 0)
      clist = tool.check(txt)
      for c in clist:
         col = 1+f+c.offset
         print(f"{n}.) Line {line}, column {col}, Rule ID: {c.ruleId}")
         print(f"Message: {c.message}")
         print(f"{c.context}")
         print(f"{' '*c.offsetInContext}{'^'*c.errorLength}")
         print()
         n = n+1




with open("preset-list","r") as f:
   presetlist = f.read().strip()
   presetlist = presetlist.split("\n")

with open("main.c", "w") as f:
   f.write("""
#include "print.c"

int main (void)
{
""")

   for ps in presetlist:
      varname = ps.lower()
      f.write(f"const EFXEAXREVERBPROPERTIES {varname} = EFX_REVERB_PRESET_{ps};\n")
      f.write(f"print_params( \"{varname}\", &{varname} );\n")

   f.write("}")



#!/usr/bin/env python3


from sys import stdout, stderr
from os.path import basename
from geometry import transf, vec
from ssys import nam2base, starmap, spob_fil, ssys_xml, vec_to_pos, pos_to_vec, naev_xml
from math import sin, pi
from minimize_angle_stretch import relax_dir
from xml_name import end_xml_name

sm = starmap()


def _key( v ):
   acc = 0
   if v[1] < 0:
      acc += 2
      v = -v
   return acc + (2 - v[0])

def mk_p( L ):
   pi = [n for n, _k in sorted(enumerate(L), key = lambda t: (_key(t[1]), t[0]))]
   where = pi.index(0)
   return pi[where:] + pi[:where]

def ssys_relax( sys, quiet = True, graph = False ):
   p = ssys_xml(sys)
   T = p['ssys']
   myname = nam2base(T['@name'])

   mapvs, sysvs, names = [], [], []
   for f in T['jumps']['jump']:
      dst = nam2base(f['@target'])
      e = f.get('pos')
      # should we require 'was_auto' ?
      if e is not None and '@was_auto' in e:
         names.append(dst)
         mapvs.append((sm[dst] - sm[myname]).normalize())
         sysvs.append(pos_to_vec(e).normalize())

   if names:
      nop = lambda v: v
      flip = nop
      pi1, pi2 = mk_p(mapvs), mk_p(sysvs)
      # more than max possible cost <= 2.0
      eps = 0.0001
      out = sys.replace('.xml', '') if graph else None

      wrn = []
      if pi1 == pi2: # same permutation -> everything is fine
         # If only two jumps, we can flip to try improving the result.
         try_flipped, try_unflipped = (len(pi1) == 2), True
      else:
         if pi1 == pi2[:1] + pi2[1:][::-1]:
            wrn += [('2', '[flipped]')]
            try_flipped, try_unflipped = True, False
         else:
            pi1 = [names[i] for i in pi1]
            pi2 = [names[i] for i in pi2]
            wrn += [('3', 'crossed: ' + ', '.join(pi1) + ' -> ' + ', '.join(pi2))]
            try_flipped, try_unflipped = True, True

      if try_unflipped:
         alpha, cost = relax_dir(sysvs, mapvs,
            eps = eps/10.0, debug = out, quiet = quiet)
      else:
         alpha, cost = 360.0, 3.0

      if try_flipped:
         outf = None if out is None else (out + '_f')
         _flip = lambda v: vec(-v[0], v[1])
         alpha_f, cost_f = relax_dir([_flip(v) for v in sysvs], mapvs,
            eps = eps/10.0, debug = outf, quiet = quiet)

         if cost_f < cost:
            alpha, cost, flip = alpha_f, cost_f, _flip

      if try_flipped and try_unflipped:
         op = 'unflipped' if flip is nop else 'flipped'
         col = '3' if (len(pi1) > 2) else '2'
         if op == 'flipped' or col == '3':
            wrn += [(col, '[better ' + op + ']')]

      if cost > 0.5:
         wrn += [('3', '[badness ' + str((int)((100*cost)/2)) + '%]')]

      if wrn:
         wrn[0] = (wrn[0][0], '"' + basename(sys) + '": ' + wrn[0][1])
         stderr.write(' '.join(['\033[3'+str(i)+'m' + j + '\033[0m' for i, j in wrn]) + '\n')
         stderr.flush()

      if abs(alpha) > eps or flip != nop:
         func = lambda x: flip(x).rotate(alpha)
         for e in T['spobs']['spob']:
            spo = naev_xml(spob_fil(nam2base(e)))
            sp = spo['spob']
            sp['pos'] = vec_to_pos(func(pos_to_vec(sp['pos'])))
            spo.save()

         for t in ['jump', 'asteroid', 'waypoint']:
            for i, e in enumerate(T[t + 's'][t]):
               d, k = (T[t + 's'][t], i) if t == 'waypoint' else (e, "pos")
               d[k] = d[k] | vec_to_pos(func(pos_to_vec(d[k])))
         p.save()
         return True
   return False

if __name__ == '__main__':
   from sys import argv, exit
   from os import fork, wait
   args = argv[1:]
   jobs = 1

   if '-h' in args or '--help' in args or not args:
      stderr.write(
         'usage:  ' + basename(argv[0]) + '[-j <n>]  [-v|-g]  <file1> ..\n'
         '  Relaxes its input xml ssys files.\n'
         '  If -j is set, uses <n> processes.\n'
         '  If -v is set, display information.\n'
         '  If -g is set, outputs the cost graph.\n'
      )
      exit(0)

   if verbose:= '-v' in args:
      args.remove('-v')

   if graph:= '-g' in args:
      args.remove('-g')

   if '-j' in args:
      i = args.index('-j')
      args.pop(i)
      try:
         jobs = int(args[i])
         args.pop(i)
      except:
         stderr.write('-j: int expected after.\n')
         exit(1)

   for i in range(jobs):
      if fork() == 0:
         for n, ssys in enumerate(args):
            if n % jobs == i:
               if ssys_relax(ssys, quiet = not verbose, graph = graph):
                  print(ssys)
                  stdout.flush()
         exit(0)
   try:
      # this one would interfere
      end_xml_name()
      while wait() != -1:
         pass
   except:
      pass



#!/usr/bin/env python3

import argparse
import re
import yaml

def main():
   ap = argparse.ArgumentParser()
   ap.add_argument('-o', '--output')
   ap.add_argument('-p', '--preamble')
   ap.add_argument('yamls', nargs='*')
   args = ap.parse_args()
   people = deduplicate(authors(args.yamls))
   with open(args.output, 'w', encoding='utf-8') as out:
      out.write(open(args.preamble, encoding='utf-8').read())
      out.writelines(map('{}\n'.format, people))

def authors(yaml_paths):
   ''' Yield all authors/contributors named in the **_LICENSE.yaml files. '''
   for path in yaml_paths:
      for small_dict in yaml.safe_load(open(path, encoding='utf-8')):
         for credited_name in small_dict:
            # The credited name is basically a free-form field. Naively attempt to break it down.
            for component in re.split(r' *[(),] *| and | \+ ', credited_name):
               component = re.sub(r'^(modified |retextured |made transparent |made |components |from |by |based on |works? |and )*', '', component, flags=re.I)
               yield component

def deduplicate(raw_authors):
   ''' Try to dedup the redundancies in the extracted authors, e.g., the same person listed
   both with and without an email address. We get these because of laziness, but from crediting
   someone once by full name and in abbreviated form when they co-author something else.'''
   l = sorted(filter(None, raw_authors), key=len)
   for i in range(len(l)-1, -1, -1):
      k = l[i].lower()
      if any(k in s.lower() for s in l[i+1:]):
         del l[i]
   l.sort(key=str.lower)
   return l


if __name__ == '__main__':
   main()



#!/usr/bin/env python3
import helper as h
N_ = lambda text: text
data = h.read()

o = data['outfit']
o['@name'] = N_('Corsair Hull Plating')

general = o['general']
del general['shortname']
general['unique'] = None
general['rarity'] = 6
general['$price'] = 1e6
general['description'] = N_("""wish I be like that lovely corsair
the prettiest of all pirate hair
be drinking at the bar
eyes green like fluorspar
irresistible buccaneer stare""")
del general['slot']['@prop_extra']

specific = o['specific']
specific['absorb']['$pri'] -= 4
specific['ew_stealth_timer'] = -10
specific['jam_chance'] = 18
specific['lua_inline_post'] = "require('outfits.core_sets.corsair_hull').init()"

data.prisec_only(sec= False)
data.save()



#!/usr/bin/env python3

import xml.etree.ElementTree as ET
from sys import argv

def fmt_kv( kv ):
   (key, value) = kv
   return key + '="' + str(value) + '"'

def writeit( T, dst ):
   def output_r( e, fp, ind = 0 ):
      li = [e.tag] + [fmt_kv(x) for x in e.attrib.items()]

      try:
         iter(e).next()
         flag = True
      except:
         flag = False

      if e.text is None and not flag:
         fp.write(' '*ind+'<'+' '.join(li)+' />\n')
      else:
         fp.write(' '*ind+'<'+' '.join(li)+'>'+e.text.rstrip())
         fst = True
         for s in e:
            if fst:
               fp.write('\n')
               fst = False
            output_r(s, fp, ind+1)
         if not fst:
            fp.write(' '*ind)
         fp.write('</'+e.tag+'>\n')

   dest = open(dst, 'w')
   output_r(T.getroot(), dest)
   dest.close()

for a in argv[1:]:
   T = ET.parse(a)
   writeit(T, a)



#!/usr/bin/env python3
# -*- encoding: utf8 -*-
# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=80:
# Licence: X/MIT
# author: Ludovic Bellière AKA. xrogaan

from collections import defaultdict
import os
import glob
import shutil
from datetime import datetime
from optparse import OptionParser

from jinja2 import Environment, FileSystemLoader
from lxml import etree
import yaml

__version__ = "1.0"

class yamlLabelReader:
   def __init__(self, stream):
      self.ydata = yaml.safe_load(stream)
      keys = set(self.ydata['shipstats'])
      labelled = set(self.ydata['statslabel'])
      for label in keys - labelled:
         print("Warning: missing label for shipstats key:", label)
      for label in labelled - keys:
         print("Notice: statslabel is orphan", label)

   def getShipStatsLabels(self, label):
      """
      Custom filter for the template enigne. Translate the xml values in
      pretty form.
      usage: {{ ship.stats|getStatsLabel }}
      """
      labels = self.ydata['shipstats']
      return labels.get(label, label+"(NOTFOUND)")

   def getStatsLabelsLabel(self, label):
      """
      Custom filter for the template engine. Explain the components.
      usage: {{ stat|getStatsLabelsLabel }}
      """
      labelsLabel = self.ydata['statslabel']
      return labelsLabel.get(label, label+"(NOTFOUND)")

class harvester:
   __xmlData=None
   __tagsBlacklist = ['sound', 'GUI']
   __tagsSortBase = ['base_type', 'class', 'price']
   __tagsSlots = ['utility', 'structure', 'weapon']

   # For updates, see ship.c:820
   __classGroup = {
      'heavy': ['carrier', 'cruiser', 'mothership'],
      'medium': ['cruise ship', 'freighter', 'destroyer', 'corvette',
                  'heavy drone', 'armoured transport']
      }

   def __init__(self, xmlPath, gfx_map):
      if self.__xmlData is None:
         self.__xmlData = glob.glob(os.path.join(xmlPath, "ships/*.xml"))

      self.ships = defaultdict(dict)
      self.shipSortBy = defaultdict(dict)
      for ship in self.__xmlData:
         ship = etree.parse(ship).getroot()
         shipName = ship.get('name')
         shipClass = ship.find('class').text

         # We only want to list player-available ships.
         if ship.find('mission') is not None:
            continue

         for details in ship.iterchildren():
            if details.tag in self.__tagsBlacklist:
               continue

            if details.tag in self.__tagsSortBase:
               if details.text not in self.shipSortBy[details.tag]:
                  self.shipSortBy[details.tag][details.text] = []

            # seems empty, ignoring. <- perhaps not a good idea
            if not details.text:
               continue

            # my, my ... You're quite empty. Let's go for the children.
            if '\n  ' in details.text:
               compiled = defaultdict(list)
               for subDetails in details.iterchildren():
                  # we're talking about slots
                  if  details.tag == "slots":
                     if subDetails.text:
                        size=subDetails.text
                     elif shipClass.lower() in self.__classGroup['heavy']:
                        size='Heavy'
                     elif shipClass.lower() in self.__classGroup['medium']:
                        size='Medium'
                     else:
                        size='Light'

                     compiled[subDetails.tag].append(size)
                  else:
                     compiled[subDetails.tag] = subDetails.text
               self.ships[shipName][details.tag] = compiled
               del(compiled)
            else:
               self.ships[shipName][details.tag] = details.text

         self.ships[shipName]['name'] = shipName
         self.ships[shipName]['gfx_comm'] = gfx_map[self.ships[shipName]['GFX']]

         if 'movement' not in self.ships[shipName]:
            self.ships[shipName]['movement'] = {'speed': 0, 'thrust': 0, 'turn': 0}

         for i in self.__tagsSortBase:
            self.store_by(i, self.ships[shipName][i], self.ships[shipName])

   def store_by(self, item, shipDetails, shipData):
      self.shipSortBy[item][shipDetails].append(shipData)

   def get_by(self, item, name=None):
      if item == 'name':
         return self.ships[name]
      return self.shipSortBy[item]

   def __iter__(self):
      return iter(self.ships.items())

class fashion:
   __xmlData = None
   __tagsBlackList = ['gfx_end','gfx','sound']
   __typeBlackList = ['map', 'gui', 'license']

   # TODO: ammo type have specific arguments (missiles have duration)
   # TODO: retrofit this code, we needs several ways to retrieve data and they
   # are all different. So, I'll discard the initial parsing and do a on
   # request walk only. groupBySlot will list only elements who's got a slot
   # subelement. getByType(name) will return a list of the specific type. Each
   # of them are unique. Here is a list: map, gui, license, ammo, modification.
   def __init__(self, xmlPath):
      if self.__xmlData is None:
         self.__xmlData = {}
         __xmlData = glob.glob(os.path.join(xmlPath, 'outfits/*/*.xml'))
         for item in __xmlData:
            basename = os.path.basename(item)
            self.__xmlData[basename] = etree.parse(item)

      self.slots = defaultdict(list)

   def _parseOutfit(self, outfitObj):
      outfitName = outfitObj.get('name')
      outfitGeneral, outfitSpecific = {}, {}

      for general in outfitObj.find('general').iterchildren():
         outfitGeneral[general.tag] = general.text

      for specific in outfitObj.find('specific').iterchildren():
         if specific.tag in self.__tagsBlackList:
            continue
         if len(specific.attrib) < 1:
            outfitSpecific[specific.tag] =  specific.text
         else:
            outfitSpecific[specific.tag] =  {
                     'attribs': specific.attrib,
                     'text': specific.text
                     }

      return {
            'name': outfitName,
            'general': outfitGeneral,
            'specific': outfitSpecific
            }

   def groupBySlots(self):
      for item in self.__xmlData.values():
         mySlot = item.findtext('general/slot')
         if mySlot is None:
            mySlot = "NA"
         self.slots[mySlot].append(self._parseOutfit(item.getroot()))

      return self.slots

   def iterSlot(self, mySlotName):
      for slotName, outfit in self.slots:
         if slotName == mySlotName:
            yield outfit['name']



if __name__ == "__main__":


   usage="Usage: %prog OUTPUTPATH"
   parser = OptionParser(usage=usage, version="%prog "+__version__,
      description="Nice looking generator for naev ships")
#   parser.add_option("-o", "--output-path", dest="output", metavar="PATH",
#      help="Path in whitch files goes.")
      parser.add_option("-t", "--template-path", dest="templates",
      default='./templates', metavar="PATH",
      help='Uses template in that PATH instead of the default one'
   )

   (cfg, arguments) = parser.parse_args()

   if len(arguments) != 1:
      parser.error("A wise man would know where to store the generated files.")
   currentPath = os.path.abspath(os.path.curdir)
   storagePath = os.path.abspath(os.path.normpath(arguments[0]))
   tplPath = os.path.abspath(os.path.normpath(cfg.templates))
   naevPath = os.path.abspath(os.path.normpath("../../dat/"))

   date = str( datetime.utcnow().strftime("%c UTC") )

   myLoader = FileSystemLoader(cfg.templates if cfg.templates else tplPath)
   labels = yamlLabelReader(open('labels.yml', 'r'))
   env = Environment(loader=myLoader)
   env.filters['getStatsLabel'] = labels.getShipStatsLabels
   env.filters['getStatsLabelsLabel'] = labels.getStatsLabelsLabel

   mediapath = storagePath + '/ships/media'
   gfx_map = {}
   os.makedirs(mediapath, 0o755, exist_ok=True)
   print('Copying image files...')
   for f in glob.glob(currentPath+'/../../*/gfx/ship/*/*_comm.*'):
      bname = os.path.basename(f)
      gfx_map[bname.rpartition('.')[0][:-5]] = bname
      shutil.copy(f, mediapath)

   # creating ships html
   myTpl = env.get_template('ships_index.html')
   yaarh = harvester(naevPath, gfx_map)
   shipIStore = os.path.normpath(storagePath + '/ships/')
   myTpl.stream(shipList=yaarh.get_by('class'), date=date).dump(shipIStore+'/index.html')
   del(myTpl)

   for (shipName, shipData) in yaarh:
      myTpl = env.get_template('ship.html')
      myPath = os.path.abspath(os.path.normpath("%s/ships/%s.html" % (storagePath,shipName)))
      myTpl.stream(shipName=shipName, shipData=shipData, date=date).dump(myPath)

   # fancy outfits
   myTpl = env.get_template('outfits_index.html')
   panty = fashion(naevPath)
   outfitsStore = os.path.normpath(storagePath + '/outfits/')
   if not os.path.exists(outfitsStore):
      os.mkdir(outfitsStore, 0o755)
   myTpl.stream(outfits=panty.groupBySlots(), date=date).dump(outfitsStore+'/index.html')

   for (slotName, outfitsList) in panty.slots.items():
      for outfitDetails in outfitsList:
         myTpl = env.get_template('outfit.html')
         myStorage = os.path.normpath("%s/outfits/%s.html") % (
               storagePath,
               outfitDetails['name']
            )
         myTpl.stream(slotName=slotName, outfitData=outfitDetails,
               date=date).dump(myStorage)

   for f in glob.glob(currentPath+'/*.css'):
      bname = os.path.basename(f)
      print('Copying css file:', bname, 'in', storagePath)
      shutil.copy(f, storagePath)



# SPDX-Licence-Identifier: GPL-3.0-or-later

from OpenGL.GL import *
from PIL import Image

def loadTexture(path):
   image = Image.open(path).convert("RGB")

   texid = glGenTextures(1)

   glBindTexture(GL_TEXTURE_2D, texid)
   glTexImage2D(GL_TEXTURE_2D, 0, GL_SRGB, image.width, image.height,
      0, GL_RGB, GL_UNSIGNED_BYTE, image.tobytes())

   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT)
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)

   return texid


def solidTexture(r, g, b):
   texid = glGenTextures(1)

   glBindTexture(GL_TEXTURE_2D, texid)
   glTexImage2D(GL_TEXTURE_2D, 0, GL_SRGB, 1, 1, 0, GL_RGB, GL_FLOAT, [r, g, b])

   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT)
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)

   return texid



import matplotlib.pyplot as plt
from matplotlib.patches import Patch

COLORMAP = plt.cm.gist_rainbow


def directedSystemPairs( problem, activated, Lfaction, systems ):
   '''Return a dictionary: {(src_sys_index, dst_sys_index): {faction_index_1, ...}}.'''
   out = {}
   sil, sjl, lanesLoc2globs = problem.internal_lanes[3:6]

   for i, jpname in enumerate(systems.jpnames):
      for target in jpname:
         out[i, systems.sysdict[target]] = set()

   for i, (sysas, jpname, lanesLoc2glob) in enumerate(zip(systems.sysass, systems.jpnames, lanesLoc2globs)):
      for jj in lanesLoc2glob:
         if activated[jj]:
            for no in sil[jj], sjl[jj]:
               no -= len(sysas)  # Adjust for index offset.
               if no >= 0:
                  out[i, systems.sysdict[jpname[no]]].add(Lfaction[jj])
   return out


def printLanes( problem, activated, Lfaction, systems ):
   '''Display the active lanes'''
   nsys = len(systems.nodess)
   nfac = len(systems.facnames)
   sil, sjl, lanesLoc2globs = problem.internal_lanes[3:6]

   sys_edge_factions = directedSystemPairs(problem, activated, Lfaction, systems)
   # Symmetrize, at least for now. (A faction might have a dead-end on the far side of a jump, making it one-way in a sense.)
   for i,j in [(i, j) for (i, j) in sys_edge_factions if i>j]:
      sys_edge_factions.setdefault((j, i), set()).update(sys_edge_factions.pop((i, j)))

   fig, (glob_ax, loc_ax) = plt.subplots(1, 2, figsize=(18, 10))
   globmap = glob_ax.scatter(systems.xlist ,systems.ylist, color='b')
   for (i, j), factions in sys_edge_factions.items():
      xij = [systems.xlist[i], systems.xlist[j]]
      yij = [systems.ylist[i], systems.ylist[j]]
      if not factions:
         glob_ax.plot(xij, yij, color='lightgrey')
      for i, f in enumerate(factions):
         nf = len(factions)
         glob_ax.plot(xij, yij, linewidth=nf, color=COLORMAP(f/nfac), dashes=(0, 5*i, 5, 5*(nf-i-1)))

   glob_ax.legend(handles=[Patch(color=COLORMAP(f/nfac), label=name) for (f,name) in enumerate(systems.facnames)])

   annot = glob_ax.annotate('', xy=(0, 0), xytext=(10, 10), textcoords='offset points', bbox={'fc': 'w'})
   annot.set_visible(False)

   def hover(event):
      vis = annot.get_visible()
      if event.inaxes == glob_ax:
         cont, ind = globmap.contains(event)
         if cont:
            n = ind['ind'][0]
            plot_system(n)
            annot.xy = globmap.get_offsets()[n]
            annot.set_text(systems.sysnames[n])
            annot.set_visible(True)
            fig.canvas.draw_idle()
         elif vis:
            annot.set_visible(False)
            fig.canvas.draw_idle()

   fig.canvas.mpl_connect('motion_notify_event', hover)

   def plot_system(i):
      nodes = systems.nodess[i]
      names = systems.sysass[i] + [f'\u2192{jn}' for jn in systems.jpnames[i]]
      lanesLoc2glob = lanesLoc2globs[i]

      loc_ax.clear()
      loc_ax.title.set_text(systems.sysnames[i])
      xlist, ylist = zip(*nodes)
      loc_ax.scatter(xlist, ylist, color='b')
      for xy, name in zip(nodes, names):
         loc_ax.annotate(name, xy=xy, xytext=(10, 10), textcoords='offset points', bbox={'fc': 'w'})

      for jj in lanesLoc2glob:
         if activated[jj]:
            no1 = sil[jj]
            no2 = sjl[jj]

            x1, y1 = nodes[no1]
            x2, y2 = nodes[no2]

            col = COLORMAP(Lfaction[jj]/nfac)
            loc_ax.plot([x1,x2], [y1,y2], color=col)


   plt.gca().set_aspect('equal')
   plt.show()



#!/usr/bin/env python3

def gammaToLinear(x):
   if x <= 0.04045:
      return x / 12.92
   return pow((x + 0.055) / 1.055, 2.4)

num_colours = 0
class Colour:
   def __init__(self, name, r, g, b, a=1.0):
      global num_colours
      num_colours += 1
      self.name = name
      self.r = gammaToLinear( r )
      self.g = gammaToLinear( g )
      self.b = gammaToLinear( b )
      self.a = a

   def write_header(self, f):
      f.write(f"extern const glColour c{self.name};\n")

   def write_source(self, f):
      f.write(f"const glColour c{self.name} = {{ .r={self.r}, .g={self.g}, .b={self.b}, .a={self.a} }};\n")

   def __lt__(self, other):
      return self.name < other.name


COLOURS = [
   Colour( "Transparent", 0.00, 0.00, 0.00, 0.00 ),
   Colour( "White",  1.00, 1.00, 1.00 ),
   Colour( "Grey90", 0.90, 0.90, 0.90 ),
   Colour( "Grey80", 0.80, 0.80, 0.80 ),
   Colour( "Grey70", 0.70, 0.70, 0.70 ),
   Colour( "Grey60", 0.60, 0.60, 0.60 ),
   Colour( "Grey50", 0.50, 0.50, 0.50 ),
   Colour( "Grey45", 0.45, 0.45, 0.45 ),
   Colour( "Grey40", 0.40, 0.40, 0.40 ),
   Colour( "Grey35", 0.35, 0.35, 0.35 ),
   Colour( "Grey30", 0.30, 0.30, 0.30 ),
   Colour( "Grey25", 0.25, 0.25, 0.25 ),
   Colour( "Grey20", 0.20, 0.20, 0.20 ),
   Colour( "Grey15", 0.15, 0.15, 0.15 ),
   Colour( "Grey10", 0.10, 0.10, 0.10 ),
   Colour( "Grey5",  0.05, 0.05, 0.05 ),
   Colour( "Black",  0.00, 0.00, 0.00 ),

   # Greens
   Colour( "DarkGreen",   0.10, 0.50, 0.10 ),
   Colour( "Green",      0.20, 0.80, 0.20 ),
   Colour( "PrimeGreen",   0.00, 1.00, 0.00 ),
   # Reds
   Colour( "DarkRed",     0.60, 0.10, 0.10 ),
   Colour( "Red",        0.80, 0.20, 0.20 ),
   Colour( "PrimeRed",    1.00, 0.00, 0.00 ),
   Colour( "BrightRed",   1.00, 0.60, 0.60 ),
   # Oranges
   Colour( "Orange",      0.90, 0.70, 0.10 ),
   # Yellows
   Colour( "Gold",       1.00, 0.84, 0.00 ),
   Colour( "Yellow",      0.80, 0.80, 0.00 ),
   # Blues
   Colour( "MidnightBlue", 0.10, 0.10, 0.4 ),
   Colour( "DarkBlue",    0.10, 0.10, 0.60 ),
   Colour( "Blue",       0.20, 0.20, 0.80 ),
   Colour( "AquaBlue",    0.30, 0.30, 0.90 ),
   Colour( "LightBlue",   0.40, 0.40, 1.00 ),
   Colour( "PrimeBlue",   0.00, 0.00, 1.00 ),
   Colour( "Cyan",       0.00, 1.00, 1.00 ),
   # Purples.
   Colour( "Purple",      0.90, 0.10, 0.90 ),
   Colour( "DarkPurple",   0.68, 0.18, 0.64 ),
   # Browns.
   Colour( "Brown",      0.59, 0.28, 0.00 ),
   # Misc.
   Colour( "Silver",      0.75, 0.75, 0.75 ),
   Colour( "Aqua",       0.00, 0.75, 1.00 ),

   Colour( "BlackHilight", 0.0, 0.0, 0.0, 0.4 ), # Hilight colour over black background

   # Outfit slot colours
   # Taken from https://cran.r-project.org/web/packages/khroma/vignettes/tol.html#muted
   Colour( "OutfitHeavy",  0.8, 0.4, 0.46 ),
   Colour( "OutfitMedium", 0.16, 0.63, 0.81 ),
   #Colour( "OutfitMedium", 0.55, 0.8, 0.93 ),
   Colour( "OutfitLight",  0.75, 0.7, 0.40 ),
   # Objects
   Colour( "Inert",      221./255., 221./255., 221./255. ),
   Colour( "Neutral",     221./255., 204./255., 119./255. ),
   Colour( "Friend",      68./255., 170./255., 153./255. ),
   Colour( "Hostile",     204./255.,  68./255., 153./255. ),
   Colour( "Restricted",   221./255., 153./255.,  51./255. ),
   # Mission Markers
   # https://packages.tesselle.org/khroma/articles/tol.html#vibrant
   Colour( "MarkerNew",      0x33/255., 0xBB/255., 0xEE/255. ),
   Colour( "MarkerComputer",   0x33/255., 0xBB/255., 0xEE/255. ),
   Colour( "MarkerLow",      0x00/255., 0x99/255., 0x88/255. ),
   Colour( "MarkerHigh",      0xEE/255., 0x77/255., 0x33/255. ),
   Colour( "MarkerPlot",      0xEE/255., 0x33/255., 0x77/255. ),
   # Radar
   Colour( "Radar_player",    0.9, 0.1, 0.9 ),
   Colour( "Radar_tPilot",    1.0, 1.0, 1.0 ),
   Colour( "Radar_tSpob",     1.0, 1.0, 1.0 ),
   Colour( "Radar_weap",      0.8, 0.2, 0.2 ),
   Colour( "Radar_hilight",   0.6, 1.0, 1.0 ),
   Colour( "Radar_scanning",   1.0, 1.0, 0.6 ),
   Colour( "Radar_viewport",   1.0, 1.0, 1.0, 0.5 ),
   # Health
   Colour( "Shield",      0.2, 0.2, 0.8 ),
   Colour( "Armour",      0.5, 0.5, 0.5 ),
   Colour( "Energy",      0.2, 0.8, 0.2 ),
   Colour( "Fuel",       0.9, 0.1, 0.4 ),

   # Deiz's Super Font Palette
   Colour( "FontRed",     1.0, 0.4, 0.4 ),
   Colour( "FontGreen",   0.6, 1.0, 0.4 ),
   Colour( "FontBlue",    0.4, 0.6, 1.0 ),
   Colour( "FontYellow",   1.0, 1.0, 0.5 ),
   Colour( "FontGrey",    0.7, 0.7, 0.7 ),
   Colour( "FontPurple",   1.0, 0.3, 1.0 ),
   Colour( "FontOrange",   1.0, 0.7, 0.3 ),
   Colour( "FontWhite",   0.95, 0.95, 0.95 ),
]
COLOURS.sort()

def write_header(f):
   f.write(f"/* FILE GENERATED BY {__file__} */")

def generate_h_file(f):
   write_header(f)

   f.write("""
#pragma once

""")
   for col in COLOURS:
      col.write_header( f )
   f.write("""
const glColour* col_fromName( const char* name );
""")

def generate_c_file(f):
   write_header(f)

   f.write(f"""
#ifdef _WIN32
#include <string.h>
#define strcasecmp _stricmp
#else
#include <strings.h>
#endif

#include <stdlib.h>
#include "colour.h"
#include "log.h"

#define NUM_COLOURS    {num_colours}

""")
   for col in COLOURS:
      col.write_source( f )

   f.write("""
typedef struct ColourSort_s {
   const char *name;
   const glColour *c;
} ColourSort_t;
const ColourSort_t col_list[NUM_COLOURS] = {
""")
   for col in COLOURS:
      f.write(f"   {{ .name=\"{col.name}\", .c=&c{col.name} }},\n")
   f.write("""
};
""")

   f.write("""
static int colour_cmp( const void *p1, const void *p2 )
{
   const ColourSort_t *c1, *c2;
   c1 = (const ColourSort_t*) p1;
   c2 = (const ColourSort_t*) p2;
   return strcasecmp( c1->name, c2->name );
}

const glColour* col_fromName( const char* name )
{
   const ColourSort_t k = { .name = name };
   const ColourSort_t *q = bsearch( &k, col_list, NUM_COLOURS, sizeof(ColourSort_t), colour_cmp );
   if (q==NULL) {
      WARN(_("Unknown colour '%s'!"),name);
      return NULL;
   }
   return q->c;
}""")

with open("colours.gen.h", "w") as colours_ggen_h:
   generate_h_file(colours_ggen_h)

with open("colours.gen.c", "w") as colours_ggen_c:
   generate_c_file(colours_ggen_c)



#!/usr/bin/env python3

import os
import subprocess
import shutil
import sys
import re

source_root = os.environ['MESON_SOURCE_ROOT']

def usage():
   print(f"usage: {os.path.basename(sys.argv[0])} [-d] (Verbose output)")
   print("DLL Bundler for Windows")
   print("This script is called by 'meson install' if building for Windows.")
   print("The intention is for this wrapper to behave in a similar manner to the bundle.py script in extras/macos.")
   print(f"usage: {os.path.basename(sys.argv[0])} [-d] (Verbose output)")
   sys.exit(1)

verbose = False

# Parse command-line arguments
args = sys.argv[1:]
while args:
   arg = args.pop(0)
   if arg == '-d':
      verbose = True
   else:
      usage()

# Get DLL search path from meson
devenv = subprocess.run([sys.executable, os.path.join(source_root, "meson.py"), "devenv", "--dump"], capture_output=True, check=True)

# WINEPATH is set in cross compilation enviornments. Check it first
devenvPath = re.search(f"WINEPATH=\"(.*);\\$WINEPATH\"",devenv.stdout.decode())
if devenvPath is not None:
   os.environ['MINGW_BUNDLEDLLS_SEARCH_PATH'] = (devenvPath.group(1) + ";" + os.environ.get('WINEPATH', default="")).replace(";", os.pathsep)

# If WINEPATH wasn't set, this is a native windows build, and we should use PATH
else:
   devenvPath = re.search(f"PATH=\"(.*){os.pathsep}\\$PATH\"",devenv.stdout.decode())
   os.environ['MINGW_BUNDLEDLLS_SEARCH_PATH'] = devenvPath.group(1) + os.pathsep + os.environ.get('PATH', default="")

# Run mingw-bundledlls to get DLL list
dll_list_cmd = [sys.executable, os.path.join(os.getenv('MESON_SOURCE_ROOT'), 'extras/windows/mingw-bundledlls/mingw-bundledlls'),
            os.path.join(os.getenv('MESON_BUILD_ROOT'), 'naev.exe')]

if verbose:
   print("Executing command:", dll_list_cmd)
   print("Working directory:", os.getcwd())

dll_list_proc = subprocess.run(dll_list_cmd, capture_output=True)
dll_list_out = dll_list_proc.stdout
dll_list_err = dll_list_proc.stderr

if verbose:
   print(dll_list_out.decode())
if verbose or dll_list_proc.returncode != 0:
   print(dll_list_err.decode())

dll_list_proc.check_returncode()

# Copy DLLs to installation directory
dll_list = dll_list_out.decode().splitlines()
for dll in dll_list:
   dll_path = os.path.join(os.getenv('MESON_INSTALL_DESTDIR_PREFIX'), os.path.basename(dll))
   shutil.copy(dll, dll_path)



#!/usr/bin/env python3


from sys import stdin, stderr, argv, exit
from geometry import bb, vec
import os
import xml.etree.ElementTree as ET



if __name__ != '__main__':
   raise Exception('This module is only intended to be used as main.')

if argv[1:]:
   stderr.write(
      'usage: ' + argv[0].split('/')[-1] + '\n'
      '  Reads a dot file on stdin, writes the graph on stdout.\n'
   )
   exit(0)

def input_blocks( it ):
   acc = ''
   for line in it:
      n = line.find(']')
      acc += line[:n]
      if n != -1:
         for c in ['\n', '\t', 3*' ', 2*' ']:
            acc = acc.replace(c, ' ')
         yield acc.strip()
         acc = ''

from graphmod import ssys_pos, ssys_jmp

dotpos = dict()
for lin in input_blocks(stdin):
   if lin.find('--') != -1:
      continue

   if (nam := lin.split(' ')[0]) in ['graph', 'edge', 'node']:
      continue

   if nam[0] == '"' and nam[-1] =='"':
      nam = nam[1:-1]
   position = lin.split('pos="', 1)[1]
   position = position[:position.find('"')].split(',')
   x, y = float(position[0]), float(position[1])
   dotpos[nam] = vec(x, y)

count = 0
bbox, oldbb = bb(), bb()
for k, v in ssys_pos.items():
   if k[0] != '_':
      oldbb += v
      if count<3 and k not in dotpos and ssys_jmp[k] != {}:
         stderr.write('"' + k + '" not found in dot output. why ?\n')
         count +=1
         if count == 3:
            stderr.write("(won't be repeated)\n")

for k in dotpos:
   dotpos[k] *= 3.0/2.0
   if k[0] != '_':
      bbox += dotpos[k]
      if k not in ssys_pos:
         stderr.write('"' + k + '" not found in ssys. Bye !\n')
         exit(-1)

again = bb()

for k, v in dotpos.items():
   dotpos[k] += oldbb.mini() - bbox.mini()
   again += dotpos[k]
   if k[0] != '_':
      ssys_pos[k] = v
stderr.write(' -> '.join([str(oldbb), str(bbox), str(again)]) + '\n')



# A small file to automatically export from Blender to STL
# This script should be called from shell via :
# blender --background --python blend_to_stl.py -- infile.gltf outfile.stl
import bpy
import os
import sys

argv = sys.argv
argv = argv[argv.index("--") + 1:]  # get all args after "--"
gltfpath = argv[0]
stlpath = argv[1]

# Extract name of file
filename = os.path.basename( bpy.data.filepath )

# Delete all
for obj in bpy.data.objects:
   obj.select_set(True)
bpy.ops.object.delete()

# Import GLTF
bpy.ops.import_scene.gltf( 'EXEC_DEFAULT', filepath=gltfpath )
# New model starts selected, so have to deselect
for obj in bpy.data.objects:
   obj.select_set(False)
# Select only body, ignore engines and other things
# TODO support for collision shapes
selected = False
scene = bpy.data.scenes[0]
for s in bpy.data.scenes:
   if s.name=="base":
      scene = s
for i in scene.objects:
   i.select_set(True)
   selected = True
if not selected:
   print(f"{gltfpath}: Failed to find any objects to select!")
   sys.exit(-1)

# Export to STL
# blender 4.0 or earlier
#bpy.ops.export_mesh.stl( "EXEC_DEFAULT",  use_selection=True, filepath=stlpath )
# blender 4.1 or later
bpy.ops.wm.stl_export( export_selected_objects=True, filepath=stlpath )



#!/usr/bin/env python3

import xml.etree.ElementTree as ET
import argparse

parser = argparse.ArgumentParser( description='Script to generate naevpedia markdown from ship XML files.')
parser.add_argument('path', metavar='PATH', type=str, help='Name of the ship XML file.')
parser.add_argument('-o', type=str, help='Output path.' )

args, unknown = parser.parse_known_args()

def tags_recursive( root, d, curpath ):
   for tag in root:
      dataname = tag.tag if curpath=='' else curpath+'/'+tag.tag
      # has children
      if len(tag):
         tags_recursive( tag, d, dataname )
      # no children
      else:
         d[dataname] = tag.text

tree = ET.parse( args.path )
root = tree.getroot()
name = root.get('name')
d = {'name':name}
tags_recursive( root, d, '' )
#print( d )

outstr = f"""---
title: "{d['name']}"
cond: "return ship.get([[{d['name']}]]):known() and not ship.get([[{d['name']}]]):tags().nonaevpedia"
---
<% s = ship.get([[{d['name']}]]) %>
"""
# We don't want any substitution below if possible
outstr += """
<%
local lg = require "love.graphics"
local luatk = require "luatk"
function mainimg ( mw )
   return luatk.newImage( nil, -10, 0, 256, 256, lg.newImage(s:gfxStore()), {
      frame = {0.05,0.05,0.05},
      bkg = {0,0,0},
   } )
end
function rightstats ( mw )
   return luatk.newText( nil, -10, 0, 256, nil, s:shipstatDesc() )
end
function wgtdescription ( mw, tw )
   return luatk.newText( nil, 0, 0, tw, nil, s:description() )
end
%>
<widget mainimg />

<widget rightstats />

# <%= s:name() %>

<widget wgtdescription />

## Properties

* **[Class](ships/classes)**:   <%= _(s:classDisplay()) %>
* **Fabricator**:   <%= _(s:fabricator()) %>
* **[Crew](mechanics/boarding)**:   <%= fmt.number(s:crew()) %>
<% if naev.player.fleetCapacity() > 0 then %>
* **[Fleet Capacity](mechanics/playerfleet)**:   <%= fmt.number(s:points()) %>
<% end %>
* **[Mass](mechanics/movement)**:   <%= fmt.f(_("{mass} {unit}"), {mass=fmt.number(s:mass()),unit=naev.unit('mass')}) %>
* **[Base Armour](mechanics/damage)**:   <%= fmt.f(_("{armour} {unit}"), {armour=fmt.number(s:armour()), unit=naev.unit('energy')}) %>
* **[Cargo Space](mechanics/cargo)**:   <%= fmt.f(_("{cargo} {unit}"), {cargo=fmt.number(s:cargo()), unit=naev.unit('mass')})%>
* **[Fuel Consumption](mechanics/hyperspace)**:   <%= fmt.f(_("{fuel} {unit}"), {fuel=fmt.number(s:fuelConsumption()), unit=naev.unit('unit')})%>
* **[Price](mechanics/credits)**:   <%= fmt.credits(s:price()) %>
<% if s:license() then %>
* **License**:   <%= _(s:license()) %>
<% end %>

## Availability

<%
   local availability = {}
   for i,sys in ipairs(system.getAll()) do
      for j,spb in ipairs(sys:spobs()) do
         if spb:known() and inlist( spb:shipsSold(), s ) then
            table.insert( availability, sys )
            break
         end
      end
   end
   table.sort( availability, function ( a, b )
      return a:jumpDist() < b:jumpDist()
   end )
%>
<% if #availability > 0 then %>
Places where the <%= s:name() %> is sold are shown in #ggreen#0.
<% function map ( mw, tw )
   local luatk_map = require "luatk.map"
   local w = math.min( mw-200, tw-20 )
   local m = luatk_map.newMap( nil, 10, 0, w, w * 9 / 16, {
      binaryhighlight = function ( sys )
         if inlist( availability, sys ) then
            return true
         end
         return false
      end
   } )
   if #availability > 0 then
      m:center( availability[1]:pos(), true )
   end
   return m
end %>
<widget map />
<% else %>
You are not aware of anywhere where the <%= s:name() %> is available for purchase.
<% end %>
"""

with open(args.o, 'w', encoding='utf-8', newline='\n') as f:
   f.write( outstr.replace('\r\n','\n') )



# python3

from outfit import outfit, naev_xml
from copy import deepcopy
from sys import stderr


def preprocess_ship(o):
   d = {}
   for n, f in o.nodes({'weapon', 'utility', 'structure'}):
      v = n[f]
      if not isinstance(v, list):
         v = [v]
      for s in v:
         what = f + '_' + s['@size'][0].upper()
         if '@prop' in s:
            if s['@prop'] == 'accessory':
               continue
            if s['@prop'][:4] == 'bio_' and s['@prop'] != 'bio_weapon':
               continue
            what += '_' + s['@prop'].split('_secondary')[0]
         d[what] = d.get(what, 0) + 1
   o.update(d)

def arg_to_naev_obj(args, combine, autostack, good):
   if combine or autostack:
      acc = []
      for i in args:
         if {i[:2], i[-2:][::-1]} & {'1x', '2x'} or '+' in i:
            stderr.write('"' + i + '" incompatible with -A/-C options -> ignored.\n')
         else:
            o = outfit(i, w= False)
            (p, s) = o.can_pri_sec()
            if p or s:
               acc.append((p, s, o.size(), o))
            else:
               stderr.write('"' + i + '" is not multicore. Ignored.\n')

      for p, _s, _S, o in acc:
         if p and o.can_alone():
            yield deepcopy(o).stack()

      if combine:
         for n, (p, s, s1, o1) in enumerate(acc):
            if p and s and o1.can_stack(o1):
                  yield (deepcopy(o1)).stack(o1)
            for q, t, s2, o2 in acc[n+1:]:
               if p and t and o1.can_stack(o2) and not (good and s1<s2):
                  yield (deepcopy(o1)).stack(o2)
               if q and s and o2.can_stack(o1) and not (good and s2<s1):
                  yield (deepcopy(o2)).stack(o1)
      elif autostack:
         for p, s, _S, o in acc:
            if p and s and o.can_stack(o):
               yield o.stack(o)
   else: # No need for copy here.
      for s in args:
         if '1x' in {s[:2], s[-2:][::-1]}:
            s = s[2:] if s[:2] == '1x' else s[:-2]
            s = s + '+'
         elif '2x' in {s[:2], s[-2:][::-1]}:
            s = s[2:] if s[:2] == '2x' else s[:-2]
            s = s + '+' + s
         s = s.split('+')
         try:
            if o := outfit(s[0], w= False):
               if len(s) == 2 :
                  if s[1].strip() == '':
                     if o.can_alone():
                        yield o.stack()
                  elif o.can_pri():
                     o2 = outfit(s[1], w= False)
                     if o2 and o2.can_sec() and o.can_stack(o2):
                        yield o.stack(o2)
               elif len(s) == 1 and o.can_alone():
                  yield o
         except Exception as e:
            if len(s) == 1:
               o = naev_xml(s[0], w= False)
               if 'ship' in o:
                  preprocess_ship(o)
               yield o
            else:
               raise e



#!/usr/bin/env python3

if __name__ != '__main__':
   raise Exception('This module is only intended to be used as main.')


from sys import argv, stderr
from graph_vaux import color_values

if argv[1:]:
   stderr.write(
      'usage: ' + argv[0].split('/')[-1] + '\n'
      '  Reads a graph file on stdin, outputs a graph on stdout.\n'
      '  Graphmod intended to be used as a preprocessing before neato is applied.\n'
      '  Mainly adds virtual edges.\n'
   )
   exit(0)

anbh = [ 'ngc11935', 'ngc5483', 'ngc7078', 'ngc7533', 'octavian',
   'copernicus', 'ngc13674', 'ngc1562', 'ngc2601', ]

del_edges = [
#   {'titus', 'vedalus'},
   {'kelvos', 'mason'},
#   {'khaas', 'diadem'},
]

new_edges = [
#  ('khaas', 'vedalus'),
   ('andres', 'mason'), ('mason', 'andres'),
   ('sunir', 'suna'), ('sunir', 'vanir'), ('sunir', 'monogram'),
]

# In the form: (from, to [, length])
virtual_edges = [
   ('flow', 'basel', 2),
   ('deneb', 'booster', 1.5),
   ('ngc4746', 'logania'),
   #('akodu', 'kenvis'),
   #('tau_ceti', 'sigur'), ('tepvin', 'carrza'),
   ('thirty_stars', 'thorndyke'),
   ('herakin', 'duros'), ('rauthia', 'tide'),
   ('hekaras', 'eneguoz'), ('seifer', 'rei'),
   ('basel', 'octantis'), ('sagittarius', 'baitas'),
   ('baitas', 'tasopa'), ('percival', 'jommel'),
   ('flow', 'katami'), ('nava', 'flow'),
   ('katami', 'eisenhorn'), ('vean', 'basel'),
   ('alpha_centauri', 'tasopa'),
   ('veses', 'protera'), ('syndania', 'stint'),
   ('sagittarius', 'alpha_centauri'), ('protera', 'scholzs_star'),
   ('ngc18451', 'felzen'), ('ngc6057', 'xeric'), ('ngc1098', 'westhaven'),
   ('ngc7061', 'kansas'), ('niger', 'kyo'),
   ('willow', 'palovi'), ('margarita', 'narousse'),
   ('porro', 'modus_manis'),
   ('tobanna', 'brumeria'),('rotide', 'tide'),
   ('padonia', 'basel'), ('ogat', 'wochii'),
   ('griffin', 'pastor'), ('ngc2948', 'ngc9017'),
   ('ngc4131', 'neexi'), ('c59', 'c14'),
   ('c43', 'c28'), ('levo', 'qellan'),
   ('nixon', 'gyrios'), ('suk', 'oxuram'),
   ('defa', 'taiomi'), ('titus', 'solene'), ('titus', 'diadem'),
   ('pike', 'kraft'), ('undergate', 'ulysses'),
   ('ngc20489', 'monogram'), ('anrique', 'adraia'),
   ('andee', 'chraan'), ('trohem', 'tepdania'),
   ('ngc14479', 'zintar'), ('pudas', 'fried'),
   ('blunderbuss', 'darkstone'), ('ekkodu', 'tarsus'),
   ('ivella', 'jommel'), ('starlight_end', 'possum'),
   ('ngc22375', 'undergate'), ('daled', 'andres'),
]

prv, prvj  = None, None
for j, i in enumerate(anbh):
   if prv is None:
      prv = i
   else:
      if prvj is not None:
         virtual_edges.append(('_'+str(prvj),    '_'+str(j)))
      prvj = j
      virtual_edges.append(('anubis_black_hole', '_'+str(j)))
      virtual_edges.append(('_'+str(j),                 prv))
      virtual_edges.append(('_'+str(j),                   i))
      prv = None

if prv is not None:
   virtual_edges.append(('_'+str(prvj),   '_'+str(prvj+2)))
   virtual_edges.append(('_'+str(prvj+2),             prv))
   virtual_edges.append(('_'+str(prvj+2),      '_'+str(1)))


virtual_edges.extend([
   ('sunir', 'botarn'), ('sunir', 'anrique'),
   ('kraft', 'kiwi')])


from graphmod import ssys_pos, ssys_jmp
from virtual_edges import add_virtual_edges

ssys_pos['sunir'] = (ssys_pos['suna'] + ssys_pos['vanir'] + ssys_pos['botarn']) / 3.0
ssys_pos.aux['sunir'] = ["default::spoiler:unused", "Su'nir"]

for sys, side in {'mason': 'north', 'c59': 'north', 'sollav': 'south'}.items():
   if ':' not in ssys_pos.aux[sys][0]:
      ssys_pos.aux[sys][0] += ':'
   for pref in ['', side]:
      if ssys_pos.aux[sys][0].find(':' + pref + 'stellarwind') == -1:
         ssys_pos.aux[sys][0] += ':' + pref + 'stellarwind'

add_virtual_edges(ssys_jmp, virtual_edges)

for v in ssys_pos:
   for e, t in ssys_jmp[v].items():
      if {v, e} in del_edges:
         t.append('fake')
      if (v, e) in new_edges:
         new_edges.remove((v, e))

for (i, j) in new_edges:
   ssys_jmp[i][j] = ['new']



#!/usr/bin/env python3
#
# This script will download and wrap meson if the current installed version
# is not at least version VERSION (which meson.build currently requires)
# if you have a newer version of meson in PATH, this wrapper isn't needed,
# but it will pass commands through all the same.
#
# To force using the downloaded Meson version, run the script with the --force-download flag:
# e.g. meson.py --force-download setup builddir
#
# Keep in mind that this script does not account for ninja-build being installed (yet)
#

import os
import subprocess
import tarfile
import urllib.request
import shutil
import sys
import argparse

MESONDIR = "meson-bin"
VERSION = "1.7.0"
PACKAGE = f"meson-{VERSION}.tar.gz"
MESON = os.path.join(MESONDIR, "meson.py")

def grab_meson():
   if not os.path.exists(MESONDIR):
      os.mkdir(MESONDIR)

   if not os.path.exists(MESON):
      print("Grabbing from online")
      urllib.request.urlretrieve(
         f"https://github.com/mesonbuild/meson/releases/download/{VERSION}/{PACKAGE}",
         PACKAGE
      )
      with tarfile.open(PACKAGE, "r:gz") as tar:
         top_level_dir = tar.getnames()[0].split('/')[0]
         members = [member for member in tar.getmembers() if member.name.startswith(top_level_dir + "/")]
         for member in members:
            member.name = member.name[len(top_level_dir) + 1:]
            if member.name:
               tar.extract(member, MESONDIR)
      os.remove(PACKAGE)
   else:
      print(f"Using cached version {VERSION}")

def run_meson(*args):
   if MESONDIR in MESON:
      subprocess.run([sys.executable, MESON] + list(args))
   else:
      subprocess.run([MESON] + list(args))

def main():
   parser = argparse.ArgumentParser(description="Wraps meson to ensure the required version is used.")
   parser.add_argument('--force-download', action='store_true', help="Force using the downloaded Meson version")
   args, meson_args = parser.parse_known_args()

   if args.force_download:
      print("Forcing use of downloaded Meson version")
      grab_meson()
   else:
      meson_path = shutil.which("meson")
      if meson_path is None:
         print("You don't have Meson in PATH")
         grab_meson()
      else:
         currentver = subprocess.check_output(["meson", "--version"]).decode().strip()
         requiredver = "1.4.0"
         if tuple(map(int, currentver.split("."))) >= tuple(map(int, requiredver.split("."))):
            print(f"Meson version is greater than or equal to {requiredver}")
            global MESON
            MESON = "meson"
         else:
            print(f"Meson version is lower than {requiredver}")
            grab_meson()

   run_meson(*meson_args)

if __name__ == "__main__":
   main()



#!/usr/bin/env python3

"""
Generates collision polygons from image or gltf files

WARNING: The description below is horribly out of date, please refer to the
argparse documentation instead which can be printed with --help.

WARNING : this script uses python 3
For needed dependancies, please see the import section

If you added a space ammo/bolt png:
Go to bottom of the file, and replace '../naev/dat/gfx/outfit/space/' and
'../naev/dat/gfx/outfit/space_polygon/' by the right paths in naev repo
Then, commertarize the line "polygonify_all_ships" and run this script
from shell (or any other interface that prints warnings).
If the script refines the polygon a lot (more than 3-4 times), there
is maybe something wrong. You'll have to add your file to special files
in "polygonify_all_outfits" function. To see what parameters to put,
read the rest of this text, or request help on discord.

If you added a ship png:
Go to bottom of the file, and replace '../naev/dat/gfx/ship/' and
'../naev/dat/gfx/ship_polygon/' by the right paths in naev repo
Then, commentarize the line "polygonify_all_outfits".
If your png has more than 8X8 sprites,
go in the function "polygonify_all_ships" and add the name of your
png in the dictionary maxNmin, along with (sx,sy,50,3,6).
If your sprite is very big (2048), replace 3,6 by 4,8
Then,run the script from shell
(or any other interface that prints warnings).
If the script refines the polygon a lot (more than 10-15 times), there
is maybe something wrong. To see what parameters to use,
read the rest of this text, or request help on discord.

Generating data for all outfits :
polygonify_all_outfits( origin_address, destination_address, overwrite )

Generating data for all ships :
polygonify_all_ships( origin_address, destination_address, overwrite )

If the third argument (overwrite) is 0, only new file will be generated
if overwrite == 1, all the polygons will be generated

Using the algorithm for one png :

1)Run polygonFromImg('address_of_my_png',sx,sy,50,3,6)
sx and sy are the nb of sprites in the picture (ex for Lancelot, its 8 and 8)
If the code doesn't give any warning, run generateXML and put the generated
file in ship_polygon or space_polygon directory.
The last two arguments are the min and max length of the polygon's faces.
Usually, I use (3,6) for ships and (2,4) for outfits.

2)If the set of points is not connex, while the png is, decrease the alpha_threshold
(4th argument of polygonFromImg, down to 1 if needed)

3)If the code says that sx or sy are wrong, check that the values you have
given are right. If they are, as written in the warning message, the error
will not be greater as 1 pixel.

4)For extremely big ships, like the diablo, it could be interesting to use
coarser dimensions like 4 and 8 for the last two arguments

5)For purposely non-connex objects (like the ripper shot), the aim is to
build a polygon that contains all the parts of the object. For that purpose,
increase the last 2 arguments up to (4,8), or even (5,10).
Remark: for these cases, a convex polygon algorithm could be preferable...

!! Very important : once you've found the right parameters to generate the
polygon from your png, add them in the dictionary maxNmin
(except if the default values are right)


Principle of the algorithm :

1 ) The transparency array is transformed into a set of points in
PointsFromImg.
Each point is adjacent to 4 pixels. In order for that point
to be activated, at least 1 of the 4 pixels must have an opacity value that
is greater than the alpha_threshold (50 is a good value for most ships)

Rem 1 : If all the pixels have opacity>=alpha_threshold, the point is not activated as
this point is for sure not on the boundary and only boundary are important
in what we want to do

Rem 2 : The value of alpha_threshold of 50 is totally arbitrary

2 ) The set of points is transformed into a polygon in polygonFromImg.
The algo picks up one of the rightmost points. This point is the starting
point. We define as well the starting direction as nearly vertical
   Then the following recurcive algo runs :
      a/ From the current point, find the next point that is at a distance
         between minlen and maxlen and that makes the minimal angle with the
         previous direction.
      b/ Compute the next direction in order to prevent going backwards.
         This direction makes the maximal angle with the direction going from
         current point to previous point, and is chosen among the points that
         are at a distance < maxlen from the previous point.
         This step is necessary in the concave parts of ships.
   After that, a few checks are performed. If the generated polygon fails,
   a new finer polygon is generated.

3 ) The polygon is simplified in simplifyPolygon.
A loop is run among the points of the polygon. Any point which angle is too
close to pi is suppressed.
"""

import numpy as np
import math
import glob
import matplotlib.pyplot as plt
import xml.etree.ElementTree as ET
import xml.dom.minidom as pretty
#from pygltflib import GLTF2
#import struct
from stl import mesh # numpy-stl
import os
import sys
import argparse
import tempfile
import subprocess
from tqdm import tqdm

# Create an array from an image
def arrFromImg( address, sx, sy ):
   buffer = plt.imread( address )

   if np.shape(buffer)[2] == 4:
      picture = buffer[:,:,3]
   elif np.shape(buffer)[2] == 2: # Black and white
      picture = buffer[:,:,1]
   else:
      print('Warning: unable to read png file')
      picture = buffer[:,:,3] # Try this, maybe it will work

   # Now see if values are in [0,1] or [0,255]
   if np.max(picture) <= 1:
      picture = 255*picture

   # Store all the different pictures in a list of matrices
   six = picture.shape[0]/sx
   siy = picture.shape[1]/sy

   if (int(six) != six) or (int(siy) != siy) :
      print(('Warning: sx = ' + str(sx) + ' or sy = ' + str(sy)
         + ' is wrong. The shape may be up to 1 pixel wrong.'))

   six = int(six)
   siy = int(siy)

   # For the case when sx or sy is wrong, all the pictures have not the
   # same size
   stax = list(range(sx))
   endx = list(range(sx))
   for i in range(sx):
      stax[i] = int( i * picture.shape[0]/sx )
      endx[i] = int( (i+1) * picture.shape[0]/sx )

   stay = list(range(sy))
   endy = list(range(sy))
   for i in range(sy):
      stay[i] = int( i * picture.shape[1]/sy )
      endy[i] = int( (i+1) * picture.shape[1]/sy )

   # Create and populate the list of matrices
   pictensor = list(range(sx*sy))
   for i in range(sx):
      for j in range(sy):
         sxy = sy*i + j
         pictensor[sxy] = picture[ stax[i]:endx[i], stay[j]:endy[j] ]

   return pictensor


# Defines points from png
def pointsFromImg( address, sx, sy, alpha_threshold ):
   if type(address)==str:
      picture = arrFromImg( address, sx, sy )
   else:
      picture = address
   npict = len(picture)

   pointsx = [] # This list of list will contain the abscissae of the points
   pointsy = []

   for p in range(npict):
      pictcur = picture[p]

      ssx = pictcur.shape[0]
      ssy = pictcur.shape[1]
      sx2 = ssx/2 # Offset value
      sy2 = ssy/2

      bufferx = [] # TODO : something to pre-allocate memory
      buffery = []

      # loop over the corners (between pixels) if at least one pixel adjacent
      # to corner is > alpha_threshold, put a point at this corner
      # + + + + + + + +
      #  0 1 1 1 0 0 0
      # + + + + + + + +
      #  0 0 1 1 1 0 0
      # + + + + + + + +

      for i in range(ssx+1): # There is 1 corner more than pixels
         for j in range(ssy+1):
            if i==0:
               if j==0:
                  if pictcur[0,0] >= alpha_threshold:
                     bufferx.append(-i+sx2) # -i because image vs coordinates
                     buffery.append(j-sy2)
               elif j==ssy:
                  if pictcur[0,ssy-1] >= alpha_threshold:
                     bufferx.append(-i+sx2)
                     buffery.append(j-sy2)
               else:
                  if pictcur[0,j-1] >= alpha_threshold or pictcur[0,j] >= alpha_threshold:
                     bufferx.append(-i+sx2)
                     buffery.append(j-sy2)

            elif i==ssx:
               if j==0:
                  if pictcur[ssx-1,0] >= alpha_threshold:
                     bufferx.append(-i+sx2)
                     buffery.append(j-sy2)
               elif j==ssy:
                  if pictcur[ssx-1,ssy-1] >= alpha_threshold:
                     bufferx.append(-i+sx2)
                     buffery.append(j-sy2)
               else:
                  if pictcur[ssx-1,j-1] >= alpha_threshold or pictcur[ssx-1,j] >= alpha_threshold:
                     bufferx.append(-i+sx2)
                     buffery.append(j-sy2)

            else:
               if j==0:
                  if pictcur[i-1,0] >= alpha_threshold or pictcur[i,0] >= alpha_threshold:
                     bufferx.append(-i+sx2)
                     buffery.append(j-sy2)
               elif j==ssy:
                  if pictcur[i-1,ssy-1] >= alpha_threshold or pictcur[i,ssy-1] >= alpha_threshold:
                     bufferx.append(-i+sx2)
                     buffery.append(j-sy2)
               else:
                  # This is the most general case. Remove points inside the domain
                  if (pictcur[i-1,j-1] >= alpha_threshold or pictcur[i-1,j] >= alpha_threshold \
                     or pictcur[i,j-1] >= alpha_threshold or pictcur[i,j] >= alpha_threshold) \
                     and not (pictcur[i-1,j-1] >= alpha_threshold and pictcur[i-1,j] >= alpha_threshold \
                     and pictcur[i,j-1] >= alpha_threshold and pictcur[i,j] >= alpha_threshold):
                     bufferx.append(-i+sx2)
                     buffery.append(j-sy2)

      pointsx.append(buffery)
      pointsy.append(bufferx) # We invert because pictures and matrix dont use the same coordinate system

   return (pointsx,pointsy)

# Simplify a polygon by removing points that are aligned with other points
def simplifyPolygon( indices, x, y, tol ):
   lim = len(indices)
   if lim < 3:
      return (indices, x, y)

   j = 0
   for i in range(lim-1): # Actually, it works as a while, with a safety bound
      xj = x[j]
      yj = y[j]
      xm = x[j+1]
      ym = y[j+1]
      x2 = x[j+2]
      y2 = y[j+2]

      th1 = math.atan2( ym-yj, xm-xj )
      th2 = math.atan2( y2-ym, x2-xm )

      # The point (xm,ym) does not make a big change in direction : kill it
      if (
            abs(th1-th2) < tol or abs(th1-th2-2*math.pi) < tol or
            abs(th1-th2+2*math.pi) < tol  # this handles the case th1~pi and th2~-pi
      ):
         del(indices[j+1])
         del(x[j+1])
         del(y[j+1])
         j -= 1

      j += 1
      if j >= len(indices)-2:
         break

   return (indices, x, y)

# Create the projections of the ship from the 3D data
def pointsFrom3D( address, slices, size, center, alpha ):
   """
   from gltflib import GLTF
   # Note doing it directly from GLTF is not working, so we use blender to drop to STL
   # Extract the mesh and the points
   gltf = GLTF().load( address )
   for s in gltf.scenes:
      if s.name=="body":
         scene = s
         break
   vertices = []
   triangles = []
   for meshid in scene.nodes:
      mesh = gltf.meshes[ meshid ]
      for primitive in mesh.primitives:
         # get the binary data for this mesh primitive from the buffer
         accessor = gltf.accessors[ primitive.attributes.POSITION ]
         bufferView = gltf.bufferViews[ accessor.bufferView ]
         buffer = gltf.buffers[ bufferView.buffer ]
         data = gltf.get_data_from_buffer_uri( buffer.uri )

         # pull each vertex from the binary buffer and convert it into a tuple of python floats
         for i in range(accessor.count):
            index = bufferView.byteOffset + accessor.byteOffset + i*12  # the location in the buffer of this vertex
            d = data[index:index+12]  # the vertex data
            v = struct.unpack("<fff", d)   # convert from base64 to three floats
            vertices.append(v)

         # unpack floats
         vertices2 = []
         for a,b,c in vertices:
            vertices2 += [a,b,c]

         # create triangles
         vertices = vertices2
         # TODO not sure why there's not a good amount of triangles...
         for i in range(0,math.floor(len(vertices)/9)*9,9):
            triangles.append(vertices[i:i+9])

   triangles = np.array(triangles).transpose()
   v0 = np.array(triangles[:][0:3])
   v1 = np.array(triangles[:][3:6])
   v2 = np.array(triangles[:][6:9])
   """

   stlfile = tempfile.NamedTemporaryFile( suffix=".stl" )
   gltftostlpy = os.path.dirname(sys.argv[0])+"/blend_gltf_to_stl.py"
   ret = subprocess.run(['blender', '--background', '--python', gltftostlpy, '--', address, stlfile.name ])
   if ret.returncode != 0:
      print("Warning: GLTF to STL export failed.")
      return None

   shipMesh = mesh.Mesh.from_file( stlfile.name )
   v0 = np.transpose(shipMesh.v0) # TODO : take the center into account
   v1 = np.transpose(shipMesh.v1)
   v2 = np.transpose(shipMesh.v2)

   # Compute x and y max and min
   xM = max( [np.amax(v0[0,:]), np.amax(v1[0,:]), np.amax(v2[0,:])] )
   xm = min( [np.amin(v0[0,:]), np.amin(v1[0,:]), np.amin(v2[0,:])] )
   yM = max( [np.amax(v0[1,:]), np.amax(v1[1,:]), np.amax(v2[1,:])] )
   ym = min( [np.amin(v0[1,:]), np.amin(v1[1,:]), np.amin(v2[1,:])] )

   # Rescale the data
   radius = max( np.max(np.linalg.norm(v0,axis=0)), np.max(np.linalg.norm(v1,axis=0)), np.max(np.linalg.norm(v2,axis=0)) )
   factor = size/radius * 0.5

   v0 = factor*v0
   v1 = factor*v1
   v2 = factor*v2

   # Rotate the stuff for any angle
   dtheta = 2*math.pi/slices

   xlist = []
   ylist = []
   for it in tqdm(range(slices), desc="Transforming model", ascii=True):
      # Rotate the points
      theta = it*dtheta + math.pi/2
      rot = np.matrix([
         [math.cos(theta), -math.sin(theta), 0],
         [math.sin(theta), math.cos(theta), 0],
         [0, 0, 1]])
      vt0 = rot * v0
      vt1 = rot * v1
      vt2 = rot * v2

      # Projection for the view
      proj = np.matrix([
         [1, 0, 0],
         [0, math.sin(alpha), math.cos(alpha)],
         [0, 0, 0]])
      vt0 = proj * vt0
      vt1 = proj * vt1
      vt2 = proj * vt2

      # Extract x and y coordinates
      x0 = vt0[0,:]
      x1 = vt1[0,:]
      x2 = vt2[0,:]
      y0 = vt0[1,:]
      y1 = vt1[1,:]
      y2 = vt2[1,:]

      xmax = max( [np.amax(x0), np.amax(x1), np.amax(x2)] )
      xmin = min( [np.amin(x0), np.amin(x1), np.amin(x2)] )
      ymax = max( [np.amax(y0), np.amax(y1), np.amax(y2)] )
      ymin = min( [np.amin(y0), np.amin(y1), np.amin(y2)] )

      # Now we create a grid of points that are inside the ship.
      # We need this regular grid because its the only way to have a
      # non-convex polygon generation algo that is guaranteed to work.
      xgrid = []
      ygrid = []
      fullDots = np.zeros( ( int(xmax)-int(xmin)+1 , int(ymax)-int(ymin)+1 ) )

      for ai, i in enumerate( range(int(xmin),int(xmax)+1) ):
         for aj, j in enumerate( range(int(ymin),int(ymax)+1) ):
            # Test if there is a triangle for which the point (i,j) is inside
            # Here, we do vector operations to speed up computations
            D1 = np.multiply(x1-i,y2-j) - np.multiply(x2-i,y1-j);
            D2 = np.multiply(x2-i,y0-j) - np.multiply(x0-i,y2-j);
            D3 = np.multiply(x0-i,y1-j) - np.multiply(x1-i,y0-j);
            D0 = D1+D2+D3;

            j1 = np.where(np.multiply(D0,D1) > 0)[1];
            j2 = np.where(np.multiply(D0,D2) > 0)[1];
            j3 = np.where(np.multiply(D0,D3) > 0)[1];
            j4 = np.intersect1d( np.intersect1d(j1,j2), j3);

            # Hell, there are flat triangles. As a consequence, > cannot
            # be replaced by >= in np.where
            if len(j4) >= 1: # point is in a triangle
               fullDots[ai,aj] = 1

      # Second loop to remove points that are inside the domain
      # (to speedup polygon generation)
      for ai, i in enumerate( range(int(xmin),int(xmax)+1) ):
         for aj, j in enumerate( range(int(ymin),int(ymax)+1) ):
            if fullDots[ai,aj] == 1:
               if (i == int(xmin) or i == int(xmax) or \
                  j == int(ymin) or j == int(ymax)):
                     # We're on the boundary : activate the point
                     xgrid.append(i)
                     ygrid.append(j)
               elif (fullDots[ai-1,aj] == 1 and fullDots[ai,aj-1] == 1\
                  and fullDots[ai+1,aj] == 1 and fullDots[ai,aj+1] == 1) :
                     # This point is inside the shape. Don't activate it
                     pass
               else:
                  xgrid.append(i)
                  ygrid.append(j)

      xlist.append(xgrid)
      ylist.append(ygrid)

      #plt.scatter(xgrid,ygrid)
      #plt.scatter(x0.tolist()[0],y0.tolist()[0])
      #break

   return (xlist, ylist, factor)

# Computes a single polygon from an image
def singlePolygonFromImg( px, py, minlen, maxlen, ppi, theta=0. ):
   npt = len(px)
   minlen2 = minlen**2
   maxlen2 = maxlen**2

   dvec = np.zeros( npt )
   dpos = np.array([math.cos(theta),math.sin(theta)])
   for i in range(npt):
      dvec[i] = np.dot( dpos, (px[i], py[i] ) )

   star   = np.argmax(dvec) # Choose the starting point
   polygon = [star] # Initialize the polygon

   # Now we do a loop
   pcur    = star
   pdir    = [math.cos(theta-1e-8),math.sin(theta-1e-8)] # Previous direction
   d02     = 0          # This value will store the distance between first and second one

   for i in range(1000): # Limit number of iterations
      xc = px[pcur]
      yc = py[pcur]

      mine = npt # Initialize the current minimal angle value
      amin = 3*math.pi

      pool = [] # Contains all the points that will be removed for next step

      for j in range(npt): # looking for current one
         if j==pcur:
            continue # Next one cannot be current one

         x  = px[j]
         y  = py[j]
         d2 = (x-xc)**2 + (y-yc)**2

         if d2 > maxlen2:
            continue # Too far away

         pool.append(j)

         if j==star and d2 < d02 and len(polygon)>7: # We made a loop
            mine = star
            break

         if d2 < minlen2 and not (j==star):
            continue # Too close

         cdir = [x-xc,y-yc]
         vprod = pdir[0]*cdir[1] - pdir[1]*cdir[0] # Vectorial product
         sprod = pdir[0]*cdir[0] + pdir[1]*cdir[1] # Scalar product
         alpha = math.atan2( vprod, sprod )      # Angle

         if alpha < 0: # We need alpha to be positive
            alpha = 2*math.pi + alpha

         if alpha < amin:
            amin = alpha
            mine = j

      if mine == star: # Loop finished
         # TODO : find a way to be sure that the last point is not after the start point
         break

      if mine < npt: # Move forward
         x = px[mine]
         y = py[mine]
         if pcur == star: # Store the distance between first and second one (to avoid getting back)
            d02 = (yc-y)**2 + (xc-x)**2
         pcur = mine
         angl = math.atan2( yc-y, xc-x  )
         adir = [ xc-x, yc-y ]

         # Seek for the angle from witch we will start for next search
         # This angle should be such that previous iterates are not reachable
         anglj = 0
         for j in pool: #polygon

            xj = px[j]
            yj = py[j]
            d2 = (x-xj)**2 + (y-yj)**2

            if d2 > maxlen2:
               continue # Not reachable anyway

            jdir = [ xj-x, yj-y ]

            vprod = adir[0]*jdir[1] - adir[1]*jdir[0] # Vectorial product
            sprod = adir[0]*jdir[0] + adir[1]*jdir[1] # Scalar product
            angla = math.atan2( vprod, sprod )      # Angle from adir to jdir
            if angla > math.pi/2: # Limit this angle
               angla = 0
            anglj = max(anglj, angla)

         angl = angl + anglj + math.pi/1e4 # Slightly increment the angle
         #print(angl)

         pdir = [ math.cos(angl), math.sin(angl) ]
         polygon.append(mine)

      else: # Did not find any value
         print('No more eligible point for polygon at sprite '+str(ppi))
         break

   ppx = [ px[i] for i in polygon ]
   ppy = [ py[i] for i in polygon ]

   return(polygon, ppx, ppy)


# Computes polygons from points
def polygonFromPoints( points, minlen, maxlen ):
   pxa = points[0]
   pya = points[1]

   npict = len(pxa)

   ppxs   = []
   ppys   = []
   polyall = []

   # List of values for minlen and maxlen. Both list should have same length
   minlist = [ 6, 5,  4, 3, 2, 1 ]
   maxlist = [ 12, 10, 8, 6, 4, 1.5 ]
   assert( len(minlist)==len(maxlist) )

   # Adapt minlist and maxlist in order to match presripted values
   minlist = list(filter(lambda x: x <= minlen, minlist))
   maxlist = list(filter(lambda x: x <= maxlen, maxlist))

   for ppi in range(npict):
      px = pxa[ppi]
      py = pya[ppi]

      theta = ppi/npict * 2*math.pi + math.pi/2

      for j in range(len(minlist)):
         stop = 1

         pplg   = singlePolygonFromImg( px, py, minlist[j], maxlist[j], ppi, theta )
         if len(pplg[1]) <= 3:
            stop = 0
            print( f"Too few points for view (only {len(ppx)}, trying again ", file=sys.stderr)
            pplg   = singlePolygonFromImg( px, py, minlist[j], maxlist[j], ppi, theta+math.pi/2 )

         polygon = pplg[0]
         ppx    = pplg[1]
         ppy    = pplg[2]
         #print( polygon, ppx, ppy )

         # Some checks
         if len(polygon)==1001:
            print('refining sprite '+str(ppi))
            stop = 0

         elif (  abs(max(ppx)-max(px)) > minlen or
               abs(min(ppx)-min(px)) > minlen or
               abs(max(ppy)-max(py)) > minlen or
               abs(min(ppy)-min(py)) > minlen):
            print('Polygon is not precise enough. Refining sprite '+str(ppi))
            stop = 0

         if stop:
            break

      polysim = simplifyPolygon( polygon, ppx, ppy, math.pi/16 ) # Simplify the polygon

      ppxs.append( polysim[1] )
      ppys.append( polysim[2] )
      polyall.append( polysim[0] )

   return (polyall, ppxs, ppys)

# Computes a polygon from an image
def polygonFromImg( address, sx, sy, alpha_threshold, minlen, maxlen ):
   points  = pointsFromImg( address, sx, sy, alpha_threshold )
   polygon = polygonFromPoints( points, minlen, maxlen )
   return (points, polygon)

# Computes a polygon from an STL
def polygonFrom3D( address, slices=120, scale=30, center=[0,0,0], alpha=math.pi/4, minlen=3, maxlen=6 ):
   points  = pointsFrom3D( address, slices, scale, center, alpha )
   xlist   = points[0]
   ylist   = points[1]
   factor  = points[2]
   polygon = polygonFromPoints( (xlist, ylist), minlen, maxlen )

   # Rescale by dividing by factor
   xlist = polygon[1]
   ylist = polygon[2]
   #xlist = [np.array(i)/factor for i in xlist]
   #ylist = [np.array(i)/factor for i in ylist]

   xpoint = points[0]
   ypoint = points[1]
   #xpoint = [np.array(i)/factor for i in xpoint]
   #ypoint = [np.array(i)/factor for i in ypoint]

   return ( (xpoint,ypoint), (polygon[0],xlist,ylist) )

# Generates a XML file that contains the polygon
def generateXML( polygon, address ):

   os.makedirs( os.path.dirname(address), exist_ok=True )

   poly = polygon[0]
   px   = polygon[1]
   py   = polygon[2]

   nb   = len(poly)

   polygons = ET.Element('polygons')
   polygons.set("num", f"{nb}")
   for i in range(nb):
      polyg = ET.SubElement(polygons,'polygon')
      polyg.set("num", f"{len(px[i])}" )
      x = ET.SubElement(polyg,'x')
      y = ET.SubElement(polyg,'y')
      x.text = ",".join(map(lambda x: str(x), px[i]))
      y.text = ",".join(map(lambda x: str(x), py[i]))

   mydata = ET.tostring(polygons, encoding="UTF-8", method="xml")
   mydata = pretty.parseString(mydata)
   mydata = mydata.toprettyxml(indent="\t",encoding="UTF-8")

   myfile = open(address, "w")
   myfile.write(mydata.decode("utf-8") )
   myfile.close()
   print(f"   Saved to {address}!")

# Generates polygon for all outfits
def polygonify_all_outfits(gfxPath, polyPath, overwrite):

   # Default parameters
   default_maxNmin = (2,4)

   # First define the parameters for special files
   maxNmin = {
      "ripperM" : (3,6)
   }

   for fileName in os.listdir(gfxPath):
      if ((fileName.endswith((".png", ".webp", ".avif")) and
            not fileName.endswith(("-end.png", "-end.webp", "-end.avif"))) and
            not fileName.startswith("beam_")):

         polyAddress = (polyPath+os.path.splitext(fileName)[0]+".xml")

         # Test if the file already exists
         if ( not overwrite and os.path.exists(polyAddress) ) :
            continue

         # Special case because of strange non-connex pixels
         if fileName == 'autocannon.png':
            print('Warning : autocannon.png was not generated')
            continue

         # Manage parameters
         lmin = default_maxNmin[0]
         lmax = default_maxNmin[1]
         if fileName in maxNmin:
            mNm = maxNmin[fileName]
            lmin = mNm[0]
            lmax = mNm[1]

         pngAddress  = (gfxPath+fileName)

         print("Generation of " + polyAddress)

         pntNplg = polygonFromImg( pngAddress, 6, 6, 50, lmin, lmax )

         polygon = pntNplg[1]
         generateXML( polygon, polyAddress )

# Generates polygon for all asteroids
def polygonify_all_asteroids( gfxPath, polyPath, overwrite ):

   # Default parameters
   default_maxNmin = (3,6,50)

   # First define the parameters for special files
   maxNmin = { "flower01" : (5,10,50) } # Actually, the algorithm automatically refines this one, so it could be skipped

   for fileName in os.listdir(gfxPath):

      polyAddress = (polyPath+os.path.splitext(fileName)[0]+".xml")

      # Test if the file already exists
      if ( not overwrite and os.path.exists(polyAddress) ) :
         continue

      # Manage parameters
      lmin = default_maxNmin[0]
      lmax = default_maxNmin[1]
      alpha_threshold = default_maxNmin[2]
      if fileName in maxNmin:
         mNm = maxNmin[fileName]
         lmin = mNm[0]
         lmax = mNm[1]
         alpha_threshold = mNm[2]

      pngAddress  = (gfxPath+fileName)

      print("Generation of " + polyAddress)

      pntNplg = polygonFromImg( pngAddress, 1, 1, alpha_threshold, lmin, lmax )
      polygon = pntNplg[1]

      """
      points  = pntNplg[0]
      plt.figure()
      plt.title(polyAddress)
      plt.scatter(points[0][0],points[1][0])
      plt.scatter(polygon[1][0],polygon[2][0])
      """

      generateXML(polygon,polyAddress)

ships = {}
def parse_ships():
   searchpath = os.path.dirname(sys.argv[0])+"/../dat/ships/**/*.xml"
   for f in glob.glob( searchpath, recursive=True ):
      root = ET.parse( f ).getroot()
      name = root.get('name')
      inherits = root.get('inherits')
      gfx = root.find( "gfx" )
      base_type = root.find( "base_type" )
      if base_type != None:
         typepath = base_type.get("path")
         if typepath == None:
            typepath = base_type.text
      if inherits == None:
         ships[ name ] = { "name": name, "gfx": gfx, "base_type": base_type, "typepath": typepath }

def polygonify_ship( filename, outpath, gfxpath, use2d=True, use3d=True ):
   root = ET.parse( filename ).getroot()
   name = root.get('name')
   inherits = root.get('inherits')
   tag, basetag, typepath = None, None, None
   if inherits != None:
      s = ships[ inherits ]
      tag = s["gfx"]
      basetag = s["base_type"]
      typepath = s["typepath"]
   def replace_none( key, val ):
      return val if val != None else key
   tag = replace_none( tag, root.find( "gfx" ) )
   basetag = replace_none( tag, root.find( "base_type" ) )
   typepath = replace_none( typepath, basetag.get("path") )
   if typepath == None:
      typepath = basetag.text
   if tag != None:
      if outpath != None:
         outname = f"{outpath}/ship/{tag.text}.xml"

      pntNplg = None
      if use3d:
         # Try 3D first
         gltfpath = f"{gfxpath}/ship3d/{typepath}/{tag.text}.gltf"
         if os.path.isfile(gltfpath):
            pntNplg = polygonFrom3D( gltfpath, scale=int(tag.get("size")) )
         else:
            print(f"{gltfpath} is not a 3D model!")
      # Fall back to image
      if use2d and pntNplg==None:
         print("Failed to find 3D model, falling back to 2D")
         imgpath = f"{gfxpath}/ship/{tag.text.split('_')[0]}/{tag.text}.avif"
         if not os.path.isfile(imgpath):
            imgpath = f"{gfxpath}/ship/{tag.text.split('_')[0]}/{tag.text}.webp"
         if not os.path.isfile(imgpath):
            imgpath = f"{gfxpath}/ship/{tag.text.split('_')[0]}/{tag.text}.png"
         sx = int(tag.get("sx")) if tag.get("sx")!=None else 8
         sy = int(tag.get("sy")) if tag.get("sy")!=None else 8
         sx, sy = sy, sx # Seems like it's flipped :/
         alpha_threshold = 50
         minlen  = 3
         maxlen  = 6
         img    = arrFromImg( imgpath, sx, sy )
         if img[0].shape[0] > 50:
            minlen = 4
            maxlen = 8
         pntNplg = polygonFromImg( img, sx, sy, alpha_threshold, minlen, maxlen )

         if outpath != None:
            outname = f"{outpath}/ship2d/{tag.text}.xml"

      # Not generated
      if pntNplg==None:
         print(f"Skipping '{filename}'...")
         return None

      # Now Generate the
      polygon = pntNplg[1]
      if outpath != None:
         generateXML( polygon, outname )
      return pntNplg

      """
      points = pntNplg[0]
      plt.figure()
      plt.title( outname )
      plt.scatter(points[0][0],points[1][0])
      plt.scatter(polygon[1][0],polygon[2][0])
      plt.show()
      """

class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter, argparse.RawDescriptionHelpFormatter):
   pass

# Run stuff
if __name__ == "__main__":
   parser = argparse.ArgumentParser(
         description = 'Wrapper for luacheck that "understands" Naev hooks.',
         formatter_class = CustomFormatter,
         epilog = f"""
Examples:

# Generate the collision polygons for the Admonisher from the Naev github repo
{sys.argv[0]} dat/ships/neutral/admonisher.xml

# Generate  the collision polygons for a ship that is fully contained in a plugin at /path/to/plugin
{sys.argv[0]} --gfxpath /path/to/plugin/gfx/ --outpath /path/to/plugin/collision/ /path/to/plugin/ships/MyShip.xml
""")
   parser.add_argument('path', metavar='PATH', nargs='+', type=str, help='Name of the ship XML file(s) to parse. Data is extracted from the ship definition about the sprites and/or 3D models.')
   parser.add_argument('--outpath', help="Path to output the polygons to.", type=str, default="assets/collision/" )
   parser.add_argument("--use2d", help="Allows the script to use 2D data (if found). Only used if 3D data is disable or not found.", default=True, action=argparse.BooleanOptionalAction )
   parser.add_argument("--use3d", help="Allows the script to use 3D data (if found).", default=True, action=argparse.BooleanOptionalAction )
   parser.add_argument("--compare", help="Computes both 2D and 3D collision polygons and compares the results visually with a plot. Only will process the first ship XML file path.", default=False, action=argparse.BooleanOptionalAction )
   parser.add_argument("--gfxpath", help="Root path which the graphic assets are located.", type=str, default="assets/gfx/" )
   parser.add_argument("--visualize", help="Whether or not the results should be visualized as a matplotlib animation. Only will process the first ship XML file path", type=bool, default=False, action=argparse.BooleanOptionalAction )
   args, unknown = parser.parse_known_args()

   #polygonify_all_outfits( args.gfxpath, args.outpath, False )

   # Comparison mode shows difference between 3D and 2D
   if args.compare:
      import matplotlib.animation as animation
      polya = polygonify_ship( args.path[0], None, False )
      polyb = polygonify_ship( args.path[0], None, True )
      def display( poly, name, idx, frame=0 ):
         frame = frame % len(poly[0][0])
         points = poly[0]
         polygon = poly[1]
         plt.subplot(1, 2, idx)
         ax = plt.gca()
         ax.set_aspect('equal', 'box')
         plt.title( name + f" ({len(polygon[0][0])} points)" )
         plt.scatter(points[0][frame],points[1][frame])
         plt.scatter(polygon[1][frame],polygon[2][frame])

      fig = plt.figure()
      display( polya, "From Image", 1 )
      display( polyb, "From GLTF", 2 )
      def animate( i ):
         plt.clf()
         display( polya, "From Image", 1, i )
         display( polyb, "From GLTF", 2, i )

      ani = animation.FuncAnimation( fig, animate, repeat=True, interval=500, cache_frame_data=False )
      plt.show()
      sys.exit(0)

   # Load up the ships
   parse_ships()

   # Normal mode we just try to process the files
   for a in args.path:
      print(f"Processing {a}...")
      try:
         poly = polygonify_ship( a, outpath=args.outpath, use2d=args.use2d, use3d=args.use3d, gfxpath=args.gfxpath )
         if args.visualize:
            import matplotlib.animation as animation
            def display( poly, name, idx, frame=0 ):
               frame = frame % len(poly[0][0])
               points = poly[0]
               polygon = poly[1]
               plt.subplot(1, 2, idx)
               ax = plt.gca()
               ax.set_aspect('equal', 'box')
               plt.title( name + f" ({len(polygon[0][0])} points)" )
               plt.scatter(points[0][frame],points[1][frame])
               plt.scatter(polygon[1][frame],polygon[2][frame])

            fig = plt.figure()
            display( poly, "Model", 1 )
            def animate( i ):
               plt.clf()
               display( poly, "Model", 1, i )

            ani = animation.FuncAnimation( fig, animate, repeat=True, interval=500, cache_frame_data=False )
            plt.show()
            sys.exit(0)
      except Exception as e:
         print( f"Failed to process '{a}': {e}", file=sys.stderr)



#!/usr/bin/env python3

if __name__ != '__main__':
   raise Exception('This module is only intended to be used as main.')


from sys import argv, stderr
from graph_vaux import color_values

if {'-h', '--help'} & set(argv[1:]):
   stderr.write(
      'usage: ' + argv[0].split('/')[-1] + '\n'
      '  Reads a graph file on stdin, outputs the same graph on stdout.\n'
      '  Writes some stats on stderr.\n'
   )
   exit(0)

from graphmod import ssys_pos
from geometry import vec

g = sum(ssys_pos.values(), vec()) / len(ssys_pos)
minx = min((x for x, _y  in ssys_pos.values()))
maxx = max((x for x, _y  in ssys_pos.values()))
miny = min((y for _x, y  in ssys_pos.values()))
maxy = max((y for _x, y  in ssys_pos.values()))

d = max(maxx-minx,maxy-miny)
f = 2.0/d
UL = vec(minx, miny)
DR = vec(maxx, maxy)
middle = 0.5 * (UL + DR)
rel_pos = lambda v: f*(v-middle)
def disp( v ):
   x,y=v
   return str((round(100*x), round(100*y)))

s= ' '.join(argv[1:])
stderr.write(s + '  window ' + disp(rel_pos(UL)) + '--' + disp(rel_pos(DR)) + '\n\n')
stderr.write(s + '  centre of mass' + disp(rel_pos(g)) + '\n')
for i in ['sol', 'gamma_polaris', 'feye', 'dvaer', 'zalek', 'aesir']:
   stderr.write(s + '  ' + i + ': ' + disp(rel_pos(ssys_pos[i])) + '\n')



#!/usr/bin/env python3

# SPDX-Licence-Identifier: GPL-3.0-or-later

import glob
import os
import math
import shutil

from OpenGL.GL import *

from obj_view.window import Window
from obj_view.parse import base_path, parse_obj
from obj_view.fb import Framebuffer
from obj_view.render import ObjProgram, RenderObject

RES = 2048
ROOT = base_path() / 'artwork/gfx/ship/3d'
OUT = base_path() / 'utils/model-view/png-exports'
shutil.rmtree(OUT, ignore_errors=True)
os.mkdir(OUT)

window = Window(0)
fb = Framebuffer(RES, RES)
glViewport(0, 0, RES, RES)

program = ObjProgram()
program.use()

for path in glob.glob(f'{ROOT}/*/*/*.obj'):
   glClearColor(0., 0., 0., 0.)
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

   try:
      print(f"Rendering '{path}'...")
      ship = parse_obj(path)
   except Exception as e:
      print(f"Error parsing '{path}': {e!r}")
      continue

   for obj in ship.values():
      obj = RenderObject(program, obj)
      program.draw(obj, RES, RES, rot=0.)
   fb.save(OUT / (os.path.basename(path)+'.png'))



from contextlib import contextmanager
import functools
import time


def timed(f):
   @functools.wraps(f)
   def wrapped(*args, **kwargs):
      with timer(f'{f.__name__} call'):
         return f(*args, **kwargs)
   return wrapped


@contextmanager
def timer(what):
   t0 = time.perf_counter()
   try:
      yield
   finally:
      t1 = time.perf_counter()
      print(f'[{t1-t0:.3f}s] {what}')



#!/usr/bin/env python3


if __name__ != '__main__':
   raise Exception('This module is only intended to be used as main.')

from sys import stderr, argv, exit

if argv[1:]:
   stderr.write(
      'usage: ' + argv[0].split('/')[-1] + '\n'
      '  Reads a graph file on stdin, outputs a graph on stdout.\n'
      '  Intended as a postprocessing for neato output.\n'
   )
   exit(0)

from geometry import vec
from graphmod import ssys_pos as pos, ssys_jmp

# average edge length.
total = 0.0
count = 0
for k in pos:
   if k[0] != '_':
      for n in [s for (s, t) in ssys_jmp[k].items() if 'tradelane' in t]:
         total += (pos[n] - pos[k]).size()
         count += 1
avg = total / count

# Position virtual systems
v = vec(avg, 0) * 0.4
for f, t, a in [
   ('beeklo',     'crimson_gauntlet',        120),
   ('anrique',    'test_of_renewal',         90),
   ('anarbalis',  'test_of_purification',    -60),
   ('churchill',  'test_of_alacrity',        0),
   ('ulysses',    'test_of_enlightenment',   135),
   ('aesir',      'test_of_devotion',        135),
]:
   pos[t] = pos[f] + v.rotate(a)



# python3

class Slst(list):
   def __call__( self, st, debug = False ):
      for T in self:
         if debug:
            print('"' + st + '"')
         (s, t, c) = (T[0], (T+('',))[1], (T+(None, None))[2])
         if c is None:
            st = st.replace(s, t)
         elif c < 0:
            st = ((st[::-1]).replace(s[::-1], t[::-1], -c))[::-1]
         else:
            st = st.replace(s, t, c)
      if debug:
         print('"' + st + '"')
      return st

   __add__= lambda *t: Slst(list.__add__(*t))

   def __getitem__( self, sl ):
      out = list.__getitem__(self, sl)
      return Slst(out) if isinstance(sl, slice) else out

   def __mul__( self, other ):
      return Slst([ (t[0], other((t+('',))[1])) + t[2:] for t in self])

   def __neg__( self ):
      return Slst([ ((t+('',))[1], t[0]) + t[2:] for t in reversed(self)])

   def __repr__(self):
      return 'Slst(' + list.__repr__(self) + ')'
   __str__ = __repr__



#!/usr/bin/env python3


#######################################################
#
#        SIM CODE
#
#######################################################

# Imports
import math
import matplotlib.pyplot as plt


def clamp( a, b, x ):
   return min( b, max( a, x ) )

def frange(start, end=None, inc=None):
   if end is None:
      start, end = 0., start + 0.
   if inc is None:
      inc = 1.
   return [i*inc+start for i in range(1+int((end-start)//inc))]


class heatsim:
   def __init__( self, shipname = "llama", weapname = "laser", simulation = [ 60., 120. ] ):
      # Sim parameters
      self.STEFAN_BOLZMANN = 5.67e-8
      self.SPACE_TEMP  = 250.
      self.STEEL_COND  = 54.
      self.STEEL_CAP   = 0.49
      self.STEEL_DENS  = 7.88e3
      self.ACCURACY_LIMIT = 500
      self.FIRERATE_LIMIT = 800
      self.shipname    = shipname
      self.weapname    = weapname
      # Sim info
      self.sim_dt      = 1./50. # Delta tick
      self.setSimulation( simulation )

      # Load some data
      self.ship_mass, self.ship_weaps = self.loadship( shipname )
      self.weap_mass, self.weap_delay, self.weap_energy = self.loadweap( weapname )

   def setSimulation( self, simulation ):
      self.simulation  = simulation
      self.sim_total   = simulation[-1]

   def loadship( self, shipname ):
      "Returns mass, number of weaps."
      if shipname == "llama":
         return 80., 2
      elif shipname == "lancelot":
         return 180., 4
      elif shipname == "pacifier":
         return 730., 5
      elif shipname == "hawking":
         return 3750., 7
      elif shipname == "peacemaker":
         return 6200., 8
      else:
         raise ValueError

   def loadweap( self, weapname ):
      "Returns mass, delay, energy."
      if weapname == "laser":
         return 2., 0.9, 4.25
      elif weapname == "plasma":
         return 4., 0.675, 3.75
      elif weapname == "ion":
         return 6., 1.440, 15.
      elif weapname == "laser turret":
         return 16., 0.540, 6.12
      elif weapname == "ion turret":
         return 42., 0.765, 25.
      elif weapname == "railgun turret":
         return 60., 1.102, 66.
      else:
         raise ValueError

   def prepare( self ):
      # Time stuff
      self.time_data   = []

      # Calculate ship parameters
      ship_kg          = self.ship_mass * 1000.
      self.ship_emis   = 0.8
      self.ship_cond   = self.STEEL_COND
      self.ship_C      = self.STEEL_CAP * ship_kg
      #self.ship_area   = pow( ship_kg / self.STEEL_DENS, 2./3. )
      self.ship_area   = 4.*math.pi*pow( 3./4.*ship_kg/self.STEEL_DENS/math.pi, 2./3. )
      self.ship_T      = self.SPACE_TEMP
      self.ship_data   = []

      # Calculate weapon parameters
      weap_kg          = self.weap_mass * 1000.
      self.weap_C      = self.STEEL_CAP * weap_kg
      #self.weap_area   = pow( weap_kg / self.STEEL_DENS, 2./3. )
      self.weap_area   = 2.*math.pi*pow( 3./4.*weap_kg/self.STEEL_DENS/math.pi, 2./3. )
      self.weap_list   = []
      self.weap_T      = []
      self.weap_data   = []
      for i in range(self.ship_weaps):
         self.weap_list.append( i*self.weap_delay / self.ship_weaps )
         self.weap_T.append( self.SPACE_TEMP )
         self.weap_data.append( [] )

   def __accMod( self, T ):
      return clamp( 0., 1., (T-500.)/600. )

   def __frMod( self, T ):
      return clamp( 0., 1., (1100.-T)/300. )

   def simulate( self ):
      "Begins the simulation."
      # Prepare it
      self.prepare()

      # Run simulation
      weap_on     = True
      sim_index   = 0
      dt          = self.sim_dt
      sim_elapsed = 0.
      while sim_elapsed < self.sim_total:
         Q_cond = 0.

         # Check weapons
         for i in range(len(self.weap_list)):
            # Check if we should start/stop shooting
            if self.simulation[ sim_index ] < sim_elapsed:
               weap_on     = not weap_on
               sim_index  += 1

            # Check if shot
            if weap_on:
               self.weap_list[i] -= dt * self.__frMod( self.weap_T[i] )
               if self.weap_list[i] < 0.:
                  self.weap_T[i]     += 1e4 * self.weap_energy / self.weap_C
                  self.weap_list[i]  += self.weap_delay

            # Do heat movement (conduction)
            Q           = -self.ship_cond * (self.weap_T[i] - self.ship_T) * self.weap_area * dt
            self.weap_T[i]  += Q / self.weap_C
            Q_cond     += Q
            self.weap_data[i].append( self.weap_T[i] )

         # Do ship heat (radiation)
         Q_rad    = self.STEFAN_BOLZMANN * self.ship_area * self.ship_emis * (pow(self.SPACE_TEMP,4.) - pow(self.ship_T,4.)) * dt
         Q        = Q_rad - Q_cond
         self.ship_T  += Q / self.ship_C
         self.time_data.append( sim_elapsed )
         self.ship_data.append( self.ship_T )

         # Elapsed time
         sim_elapsed += dt;

   def save( self, filename ):
      "Saves the results to a file."
      with open( self.filename, 'w' ) as f:
         for time, ship, weap in zip(self.time_data, self.ship_data, self.weap_data):
            f.write( f'{time} {ship_data}')
            for wj in weap:
               f.write( f' {wj}' )
            f.write( '\n' )

   def display( self ):
      print("Ship Temp:", hs.ship_T, "K")
      for i, T in enumerate(hs.weap_T):
         print(f"Outfit[{i}] Temp: {T} K")


   def plot( self, filename=None ):
      plt.figure()

      # Plot 1 Data
      plt.subplot(211)
      plt.plot( self.time_data, self.ship_data, '-' )

      # Plot 1 Info
      plt.axis( [0, self.sim_total, 0, 1100] )
      plt.title( 'NAEV Heat Simulation ('+self.shipname+' with '+self.weapname+')' )
      plt.legend( ('Ship', 'Accuracy Limit', 'Fire Rate Limit'), loc='upper left')
      plt.ylabel( 'Temperature [K]' )
      plt.grid( True )

      # Plot 1 Data
      plt.subplot(212)
      plt.plot( self.time_data, self.weap_data[0], '-' )
      plt_data = [self.ACCURACY_LIMIT] * len(self.weap_data[0])
      plt.plot( self.time_data, plt_data, '--' )
      plt_data = [self.FIRERATE_LIMIT] * len(self.weap_data[0])
      plt.plot( self.time_data, plt_data, '-.' )

      # Plot 2 Info
      plt.axis( [0, self.sim_total, 0, 1100] )
      plt.legend( ('Weapon', 'Accuracy Limit', 'Fire Rate Limit'), loc='upper right')
      plt.ylabel( 'Temperature [K]' )
      plt.xlabel( 'Time [s]' )
      plt.grid( True )
      if filename is None:
         plt.show()
      else:
         plt.savefig( filename )


if __name__ == "__main__":
   print("NAEV HeatSim\n")
   shp_lst = { 'llama' : 'laser',
               'lancelot' : 'ion',
               'pacifier' : 'laser turret',
               'hawking' : 'ion turret',
               'peacemaker' : 'railgun turret' }

   for shp,wpn in shp_lst.items():
      hs = heatsim( shp, wpn, (60., 120.) )
      #hs = heatsim( shp, wpn, frange( 30., 600., 30. ) )
      hs.simulate()
      hs.plot( f'{shp}_{wpn}_60_60.png' )
      hs.setSimulation( (30., 90.) )
      hs.simulate()
      hs.plot( f'{shp}_{wpn}_30_60.png' )
      hs.setSimulation( (30., 90., 120., 180.) )
      hs.simulate()
      hs.plot( f'{shp}_{wpn}_30_60_30_60.png' )
      print( '  ', shp, 'with', wpn, 'done!' )



#!/usr/bin/env python3
import helper as h
N_ = lambda text: text
data = h.read()

o = data['outfit']
o['@name'] = N_('Corsair Engine')

general = o['general']
del general['shortname']
general['unique'] = None
general['rarity'] = 6
general['$price'] = 1e6
general['description'] = N_("""there once was a big mighty corsair
who was said to be the finest heir
of some noble great house
almost tied as a spouse
before they quickly took to the air""")
del general['slot']['@prop_extra']

specific = o['specific']
del specific['time_mod']
specific['jump_distance'] = 25
specific['lua_inline_post'] = "require('outfits.core_sets.corsair_engine').init()"

data.prisec_only(sec= False)
data.save()



#!/usr/bin/env python3

"""
An elementTree-based implementation of xmltodict. 10% slower than pure elementTree,
but 2x faster than xmltodict with more functionality and correct output
(empty element folding, new lines, no useless xml 1.0 header, etc.).

Used as follows:
   o = naev_xml('input.xml')
   # Here o is just the regular xmltodict-generated dictionary, except:

   #  1. the subtrees you insert are deep-copied. This prevents accidentally turning
   #     your tree into a dag. The "downside":
   t = {'a': '1', 'z': '26'}
   o['new_child'] = t
   t['a'] = '2'
   # !!! here o['new_child']['a'] is still '1'

   #  2. You can use an additional '$' character in you tag names:
   o['nums'] = {'fst': '1', 'snd': '2.0', 'thd': '3.14', 'fth': 'zero'}
   # in that case:
   # o['nums']['$fst'] -> 1
   # o['nums']['$snd'] -> 2
   # o['nums']['$thd'] -> 3.14
   # o['nums']['$fth'] -> exception (attempt to use non-value str as a value)
   o['nums']['$fst'] = 1     # saves 1
   o['nums']['$snd'] = 2.0   # saves 2
   o['nums']['$thd'] = 3.14  # saves 3.14
   o['nums']['$fth'] = '1.0' # raises an exception
   # this allows:
   # o['$speed'] *= 1.2
   # o['$thing'] = 2.0 * 0.5 # will save 1 instead of 1.0

   #  3. At init, you can specify the r and w flags (both True by default).
   #     You get errors in the following cases:
   #      - r is True and the input file can't be openened
   #      - w is False and you attempt to save
   #     You get warnings in the following cases:
   #      - w is true the dict gets garbage collected while with unsaved changes.
   #      - the dict gets saved while its destination is already up-to-date.

   # This optional call allows to change the destination.
   # If not called, save destination is the same as input.
   o.save_as('output.xml')
   o.save()
   # xmltodict's unparse was missing a final newline and unable to fold empty
   # tags. This has been fixed.
"""

import xml.etree.ElementTree as ET
from sys import stderr, stdout
from os import devnull

def _numify( s ):
   try:
      f = float(s)
      return int(f) if round(f) == f else f
   except:
      pass

class _xml_list( list ):
   def __init__( self, parent, key, L=[] ):
      self._parent = parent
      self._key = key
      list.__init__(self, [_xml_ify(i, parent, key) for i in L])

   def _change( self ):
      self._parent._change()

   def __setitem__( self, key, value ):
      if isinstance(key, slice):
         list.__setitem__(self, key, _xml_list(self._parent, self._key, value))
      else:
         list.__setitem__(self, key, _xml_ify(value, self._parent, self._key))
      self._change()

   def insert( self, index, elt ):
      list.insert(self, index, _xml_ify(elt, self._parent, self._key))
      self._change()

   def append( self, elt ):
      list.append(self, _xml_ify(elt, self._parent, self._key))
      self._change()

   def extend( self, other ):
      list.extend(self, _xml_list(self._parent, self._key, other))
      self._change()

   __iadd__ = extend

   def clear( self ):
      self._change()
      list.clear(self)

   def pop( self, index= -1 ):
      self._change()
      return list.pop(self, index)

   def remove( self, val ):
      self._change()
      list.remove(self, val)

   def reverse( self ):
      self._change()
      list.reverse(self)

   def sort( self, **args ):
      self._change()
      list.sort(self, **args)

   def __delitem__( self, item ):
      self._change()
      list.__delitem__(self, item)

class xml_node( dict ):
   def __init__ ( self, mapping, parent= None, key= None ):
      self._parent = parent
      self._key = key
      if isinstance(mapping, xml_node):
         self.attr = mapping.attr.copy()
      else:
         self.attr = {}
      for k, v in mapping.items():
         self[k] = v

   def _change( self ):
      if self._parent is None:
         self._uptodate = False
      else:
         self._parent._change()

   def tag( self ):
      return self._key

   def parent( self ):
      return self._parent

   def __contains__ ( self, key ):
      key = key[1:] if key[:1] == '$' else key
      return key in self.attr if key[:1] == '@' else dict.__contains__(self, key)

   def __getitem( self, key ):
      return self.attr[key] if key[:1] == '@' else dict.__getitem__(self, key)

   def __getitem__( self, key ):
      if isinstance(key, str) and key[0] == '$':
         return _numify(self.__getitem(key[1:]))
      else:
         return self.__getitem(key)

   def __setitem__( self, key, val ):
      if isinstance(key, str) and key[0] == '$':
         val = _numify(val)
         if val is None:
            raise ValueError(str(val) + ' is not a number.')
         key = key[1:]
      elif isinstance(val, list):
         val = _xml_list(self, key, val)
      elif isinstance(val, dict):
         val = xml_node(val, self, key)
      if key[:1] == '@':
         self.attr[key] = val
      else:
         dict.__setitem__(self, key, val)
      self._change()

   def __delitem__( self, key ):
      if key[:1] == '@':
         del self.attr[key]
      else:
         dict.__delitem__(self, key)
      self._change()

   def __repr__( self ):
      return dict.__repr__(self.attr | self)

   __str__ = __repr__

   def clear(self):
      dict.clear(self)
      self.attr.clear()
      self._change()

   # Do these make any sense in this context ?
   popitem = None
   pop = None

   # update does not rely on setitem, contrary to the doc
   # This is the implementation corresponding to the doc.
   def update( self, *E, **F ):
      if E:
         E,*_ = E
         if hasattr(E, 'keys'):
            for k in E.keys():
               self[k] = E[k]
         else:
            for k, v in E:
               self[k] = v
      for k in F:
         self[k] = F[k]

   def __ior__( self, other ):
      self.update(other)
      return self

   def keys():
      return self.attr.keys() + list.keys(self)

   def setdefault( self, key, default= None ):
      self._change()
      return dict.setdefault(self, key, default)


def _xml_ify( elt, par, k ):
   return xml_node(elt, par, k) if isinstance(elt, dict) else elt

class _trusted_node( xml_node ):
   def __init__ ( self, attr, parent= None, key= None ):
      self.attr = {('@' + k): v for k, v in attr.items()}
      dict.__init__(self, {})
      self._parent = parent
      self._key = key

def _parse( node, par, key ):
   d = _trusted_node(node.attrib, par, key)

   for e in node:
      if dict.__contains__(d, e.tag):
         crt = dict.__getitem__(d, e.tag)
         if not isinstance(crt, list):
            l = _xml_list(d, e.tag)
            list.append(l, crt)
            dict.__setitem__(d, e.tag, l)
         list.append(dict.__getitem__(d, e.tag), _parse(e, d, e.tag))
      else:
         dict.__setitem__(d, e.tag, _parse(e, d, e.tag))

   t = node.text
   t = t and t.strip() or None
   if d == {} and d.attr == {}:
      return t
   if t:
      dict.__setitem__(d, '#text', t)
   return d

def _unparse_elt( v, k, indent):
   out = indent*' ' + '<' + k
   content = ''

   if isinstance(v, xml_node):
      for ka, va in v.attr.items():
         out += ' ' + ka[1:] + '="' + str(va).replace('&', '&amp;') + '"'
      if dict.__contains__(v, '#text'):
         content = v['#text']
      elif sub := unparse(v, indent+1):
         content = '\n' + sub + indent*' '
   elif v or v == 0:
      content = str(v).replace('&', '&amp;')

   if content:
      out += '>' + content + '</' + k + '>\n'
   else:
      out += '/>\n'
   return out

def unparse( d , indent= 0 ):
   out = ''
   for k, v in d.items():
      if not isinstance(v, list):
         v = [v]
      out += ''.join((_unparse_elt(ve, k, indent) for ve in v))
   return out

class naev_xml( xml_node ):
   def __init__( self, fnam= None, r= True, w= True):
      self.short = None
      if fnam is None:
         r = False
         fnam = devnull
      self._uptodate = r
      self.w, self.r = w, r
      if type(fnam) != type('') or ('xml' not in fnam.split('.') and fnam != devnull):
         raise Exception('Invalid xml filename ' + repr(fnam))
      self._filename = fnam

      self.short = False
      _trusted_node.__init__(self, {}, None, None)
      if r and self._filename != devnull:
         try:
            P = ET.parse(self._filename)
         except FileNotFoundError as e:
            raise e
         # Can also raise:
         #  - xml.etree.ElementTree.ParseError: ...
         except Exception as e:
            raise Exception('Invalid xml file ' + repr(self._filename)) from e

         T = P.getroot()
         try:
            dict.__setitem__(self, T.tag, _parse(T, self, T.tag))
         except:
            raise Exception('Invalid xml file ' + repr(self._filename))

   def save( self, if_needed= False ):
      if not self.w:
         raise Exception('Attempt to save read-only ' + repr(self._filenam))
      if self._uptodate:
         if if_needed:
            return False
         else:
            stderr.write('Warning: saving unchanged file ' + repr(self._filename) + '.\n')

      if self._filename == '-':
         stdout.write(unparse(self))
      else:
         with open(self._filename, 'w') as fp:
            fp.write(unparse(self))

      self._uptodate = True
      return True

   def save_as( self, filename= None ):
      filename = filename or devnull
      if not self.w:
         raise Exception('Attempt to save read-only ' + repr(self._filenam))
      if filename == self._filename:
         stderr.write('Warning: save destination was already ' + repr(self._filename) + '.\n')
      else:
         self._uptodate = filename == devnull
         self._filename = filename

   def find( self, key, ref = False):
      for d, k in self.nodes(lookfor = { key }):
         return d if ref else d[k]

   @staticmethod
   def _nodes( node , lookfor ):
      for k, v in node.items():
         if not lookfor or k in lookfor:
            if _numify(v) is None:
               yield node, k
            else:
               yield node, '$' + k
         if isinstance(v, dict):
            yield from naev_xml._nodes(v, lookfor)

   def nodes( self, lookfor= None ):
      return self._nodes(self, lookfor)

   @staticmethod
   def _to_dict( node ):
      for k, v in node.items():
         if isinstance(v, dict):
            yield from naev_xml._to_dict(v)
         elif (vv := _numify(v)) is not None:
            yield k, vv

   def to_dict( self ):
      return dict(self._to_dict(self))

   def name( self ):
      return next(iter(self.values()))['@name']

   def shortname( self ):
      if not self.short:
         if (res := self.find('shortname')) is None:
            res = self.name()
         if res.split(' ')[-1] == 'Engine':
            res = ' '.join(res.split(' ')[:-1])
         self.short = res
      return self.short

   def touch( self ):
      self._uptodate = False

   def changed( self ):
      return not self._uptodate

   def __del__( self ):
      if self.w and not self._uptodate and self._filename != devnull:
         stderr.write('Warning: unsaved file ' + repr(self._filename) + ' at exit.\n')

def xml_parser( constr, qualifier= '' ):
   from pathlib import Path
   from sys import argv
   from os import path
   from pathlib import Path

   args = argv[1:]
   if inplace := '-i' in args:
      args.remove('-i')

   if clone := '-c' in args:
      args.remove('-c')
      src, dst = (args + [None, None])[:2]
      args = args[2:]

   if args == [] or '-h' in args or (clone and inplace):
      stderr.write(
         'usage: ' + argv[0].split('/')[-1] + ' [ -i | (-c <src_path> <dst_path>)] [ file.xml.. ]\n'
         '  Reads a ' + qualifier + 'xml file, formats it and outputs the result.\n'
         '  If -i (in-place) is set, does it in place.\n'
         '  If -c (clone) is set, for each <file.xml>, produces an uncommented version\n'
         '  that is written to a file that has the same position in <dst_path> that\n'
         '  <file.xml> has in <src_path>.\n'
      )
      exit(0)
   for arg in args:
      x = constr(arg)
      if inplace:
         x.touch()
      elif clone:
         if (rel := path.relpath(path.realpath(arg), src))[:2] == '..':
            stderr.write('\033[31merror:\033[0m file ' + repr(arg) + ' not in ' + repr(src) +'\n')
            exit(1)
         oarg = path.join(dst, rel)
         Path(path.split(oarg)[0]).mkdir(parents= True, exist_ok= True)
         x.save_as(oarg)
      else:
         x.save_as('-')
      x.save()

if __name__ == '__main__':
   xml_parser(naev_xml)



#!/usr/bin/env python3


import os
from sys import stderr, exit
from ssys import nam2base, getpath, PATH, naev_xml, ssys_xml, vec_to_pos
from graph_vaux import ssys_others, ssys_nam



if __name__ != '__main__':
   raise Exception('This module is only intended to be used as main.')

from sys import argv, exit, stdout, stderr, stdin

help_f = '-h' in argv or '--help' in argv[1:]
if help_f or (argv[1:] and do_write):
   msg = lambda s: (stdout if help_f else stderr).write(s + '\n')
   DOC = [
      'usage:  ' + os.path.basename(argv[0]),
      '  Updates ssys/*.xml according to the graph provided in input.'
   ]
   for l in DOC:
      msg(l)
   exit(0 if help_f else 1)

def new_ssys( name, basenam, ssys_pos, jmp ):
   xml = naev_xml(name, r= False)
   Nam = ssys_nam(ssys_pos, basenam)
   fast_small_ship = 400
   fuel_regen_factor = 1 + 2
   rad = 100.0 / fuel_regen_factor * fast_small_ship / 2 # 1 diameter == 2 radiuses :-)
   if Nam and nam2base(Nam) != basenam:
      stderr.write('Warning: basename "' + basenam + '" does not match provided name "' + Nam  + '"\n')
   mk_jump = lambda dst, aux: {
      '@target': ssys_nam(ssys_pos, dst),
      'autopos': {},
      'hide': 1,
      'tags': {'tag': list(set(aux) - {'new'})},
   }
   xml['ssys'] = {
      '@name': Nam or ' '.join([s[0].upper() + s[1:] for s in basenam.split('_')]),
      'general': {'radius': rad, 'spacedust': 300, 'interference': 0},
      'pos': vec_to_pos(v),
      'spobs': {},
      'jumps': {'jump': [mk_jump(k, v) for k, v in jmp.items()]},
      'asteroids': {},
      'tags': {'tag': list(set(ssys_others(ssys_pos, basenam)) - {'new', 'update'}) },
   }
   if 'stellarwind' in ssys_others(ssys_pos, basenam):
      xml['ssys']['general'] |= {
         'background': 'stellarwind',
         'map_shader': 'stellarwind_map.frag',
         'features': '#bStellar Wind (1 fuel regen)#0'
      }
   return xml

from graphmod import ssys_pos, ssys_jmp, no_graph_out

no_graph_out()

for n, v in ssys_pos.items():
   name = os.path.join(PATH, 'ssys', n + '.xml')
   try:
      xml = ssys_xml(name)
   except FileNotFoundError:
      xml = None
   xml = new_ssys(name, n, ssys_pos, ssys_jmp[n]) if xml is None else xml
   xml['ssys']['pos'] = vec_to_pos(v)
   xml.save()



#!/usr/bin/env python3

import argparse
import csv
import glob
import xml.etree.ElementTree as ET


class Balancer:

   def __init__(self):
      self.ignore = []

   def __tags_recursive( self, root, d, fields, ignore, curpath ):
      for tag in root:
         if tag.tag in ignore:
            continue
         dataname = tag.tag if curpath=='' else curpath+'/'+tag.tag
         # has children
         if len(tag):
            self.__tags_recursive( tag, d, fields, ignore, dataname )
         # no children
         else:
            d[dataname] = tag.text
            fields.update( [dataname] )

   def __xml2dict( self, searchpath, ignore=[] ):

      # parse the xml files
      fields  = set()
      data   = {}
      for s in searchpath.split(','):
         for filename in glob.glob( s, recursive=True ):
            print(filename)
            root = ET.parse( filename ).getroot()

            name = root.get('name')
            d = {'name':name}
            self.__tags_recursive( root, d, fields, ignore, '' )
            data[name] = d

      fields = list(fields)
      fields.sort()
      fields.insert(0,'name')

      return fields, data

   def __dict2xml( self, data, searchpath ):
      for s in searchpath.split(','):
         for filename in glob.glob( s, recursive=True ):
            tree = ET.parse( filename )
            root = tree.getroot()
            name = root.get('name')

            # iterate over items
            if data.get(name)==None:
               continue

            for key,val in data[name].items():
               # skip name and empty cells
               if key=='name' or val=='' or key=='':
                  continue
               tags = root.findall(key)
               if len(tags) > 1:
                  print(f"Found duplicate tag '{key}' in '{val}'. Removing!")
                  for i in range(1,len(tags)):
                     root.remove( tags[i] )
               tag = root.find(key)
               if tag==None:
                  # have to add new tag
                  new_tag = ET.SubElement(root,key)
                  new_tag.text = val
               else:
                  # update existing value
                  tag.text = val
            # save the file
            self.__write_tree( tree, filename )

   def __write_tree( self, tree, ofile ):
      # save the file
      tree.write( ofile, encoding="UTF-8", xml_declaration=True )
      # With python 3.9 the line below should work and make everything pretty ;)
      #ET.indent(tree).write( ofile, encoding="UTF-8", xml_declaration=True )


   def xml2csv( self, csvfile ):
      fields, data = self.__xml2dict( searchpath=self.searchpath, ignore=self.ignore )

      # save the outfit data
      with open(csvfile,'w') as f:
         w = csv.DictWriter( f, fieldnames=fields, quotechar='"', quoting=csv.QUOTE_ALL )
         w.writeheader()
         for n,d in data.items():
            w.writerow(d)

   def csv2xml( self, csvfile ):
      # load the outfit data
      data = {}
      with open(csvfile,'r') as f:
         r = csv.DictReader( f, quotechar='"' )
         for row in r:
            data[ row['name'] ] = row

      self.__dict2xml( data, self.searchpath )


class ShipBalancer(Balancer):
   def __init__(self):
      super().__init__()
      self.searchpath = "dat/ships/*.xml"
      self.ignore = "slots"


class OutfitBalancer(Balancer):
   def __init__(self):
      super().__init__()
      self.searchpath = "dat/outfits/**/*.xml,build/dat/outfits/**/*.xml"


if __name__ == '__main__':
   parser = argparse.ArgumentParser(description='Tool to save Naev xml file data to an csv file and viceversa.')
   parser.add_argument('mode', choices=['outfit','ship'], help="Either 'outfit' to process outfits or 'ship' to process ships.")
   parser.add_argument('readwrite', choices=['w','r'], help="Either 'r' to read from the naev data or 'w' to write to the naev data.")
   parser.add_argument('filename', type=str, help="CSV file to either read from or write to.")
   args = parser.parse_args()

   if args.mode=='outfit':
      balancer = OutfitBalancer()
   else:
      balancer = ShipBalancer()

   if args.readwrite=='w':
      print( 'Writing to XML files from %s!' % args.filename )
      balancer.csv2xml( args.filename )
   else:
      print( 'Reading from XML files to %s!' % args.filename )
      balancer.xml2csv( args.filename )



#!/usr/bin/env python3


if __name__ != '__main__':
   raise Exception('This module is only intended to be used as main.')

from sys import stderr, argv, exit

if argv[1:]:
   stderr.write(
      'usage: ' + argv[0].split('/')[-1] + '\n'
      '  Reads a graph file on stdin, outputs a graph on stdout.\n'
      '  Horizontally stretches the north.\n'
   )
   exit(0)

from geometry import vec, bb
from graphmod import ssys_pos as pos

box = bb()
for (_, v) in pos.items():
   box += v

STRETCH = 0.1
def f(x):
   x = 1.0 - x
   if x > 0.5:
      return 1.0
   elif x < 0.25:
      return 1.0 + STRETCH
   else:
      x = x*4.0 - 1.0
      return 1.0 + STRETCH * (1.0 - x)

anbh = { 'ngc11935', 'ngc5483', 'ngc7078', 'ngc7533', 'octavian',
   'copernicus', 'ngc13674', 'ngc1562', 'ngc2601', 'zied'}
for (k, v) in pos.items():
   if k not in anbh:
      t= (v[1] - box.miny) / (box.maxy - box.miny)
      pos[k] = vec(pos[k][0] * f(t), pos[k][1])



#!/usr/bin/env python3
# This file reads missions data and maps it

# Tier rules so far:

# 0: automatic
# 1: no fight, basic exploration
# 2: occasionnal fight, advanced flying skills needed
# 3: fight needed, light ship and no special skills needed
# 4: Heavy ship needed or tricky combat
# Above: plot-related

# A mission that requires another mission to be done must be of > tier, or no indicated tier at all

import xml.etree.ElementTree as ET
import pygraphviz as pgv
import glob
import os
import argparse

# Should we represent different campaigns and different tiers?
parser = argparse.ArgumentParser(description='Tool to display the relationships between the missions in Naev.')
parser.add_argument('--campaigns', metavar='c', type=bool, default=False, help="Ignore the campaign relationships when creating the graph." )
parser.add_argument('--tiers', metavar='t', type=bool, default=True, help="Ignore the tier relationships when creating the graph." )
args = parser.parse_args()
ignore_camp = not args.campaigns
ignore_tier = not args.tiers

tierNames = [
   "Automatic",
   "Basic piloting",
   "Advanced piloting",
   "Basic combat",
   "Advanced combat",
   "Plot-related",
]


i = 0
namdict = {} # Index from name
names = []
dones = []
uniques = [] #
extra_links = [] # List of links given by extra info
meta_nodes = [] # List of meta nodes (useful to control how they're displayed)
camp = [] # Name of the campaigns the missions are in
tierL = []

directory = os.path.split(os.getcwd())
if directory[1] == 'utils':
   prefix = '..'
elif directory[1] == 'naev':
   prefix = '.'
else:
   print("Failed to detect where you're running this script from\nPlease enter your path manually")

def add_notes( name, parent, base_campaign, campaigns_list, tiers_list ):
   notes = parent.find('notes')

   if notes != None:
      campaign = notes.find("campaign")
      tier = notes.find("tier")
   else:
      campaign = None
      tier = None

   if campaign == None:
      campTxt = base_campaign
   else:
      campTxt = campaign.text

   if tier == None:
      tierV = None
   else:
      tierV = int(tier.text)

   campTxt = 'cluster: '+campTxt
   campaigns_list.append(campTxt)
   tiers_list.append(tierV)

   if notes == None:
      return

   done_misn = notes.findall('done_misn')
   for dm in done_misn:
      previous = 'Misn: '+dm.attrib['name']
      if dm.text == None:
         dm.text = ""
      extra_links.append( (previous, name, dm.text)  )

   done_evt = notes.findall('done_evt')
   for dm in done_evt:
      previous = 'Evt: '+dm.attrib['name']
      if dm.text == None:
         dm.text = ""
      extra_links.append( (previous, name, dm.text)  )

   provides = notes.findall('provides')
   for p in provides:
      nextt = p.attrib['name']
      if p.text == None:
         p.text = ""
      extra_links.append( (name, nextt, p.text)  )
      if (not (nextt in meta_nodes)):
         meta_nodes.append((nextt,campTxt))

   requires = notes.findall('requires')
   for r in requires:
      previous = r.attrib['name']
      if r.text == None:
         r.text = ""
      extra_links.append( (previous, name, r.text)  )
      if (not (previous in meta_nodes)):
         meta_nodes.append((previous,campTxt)) # TODO: there will be conflicts between differnent requires


# Reads all the missions
for missionfile in glob.glob( prefix+'/dat/missions/**/*.lua', recursive=True ):
   try:
      print(missionfile)

      with open(missionfile,'r') as f:
         buf = f.read()
      if buf.find('</mission>') < 0:
         continue
      p = buf.find('--]]')
      if p < 0:
         continue
      xml = buf[5:p]

      tree = ET.ElementTree(ET.fromstring(xml))
      misn = tree.getroot()

      name = 'Misn: '+misn.attrib['name']
      names.append(name)
      namdict[name] = i

      done = misn.find('done')  # TODO: I guess findall is needed if there are more than one
      dones.append(done)

      unique = misn.find('unique')
      if unique == None:
         uniques.append(False)
      else:
         uniques.append(True)

      # Read the notes
      add_notes( name, misn, "Generic Missions", camp, tierL )

      i += 1
   except:
      pass

namdictE  = {} # Index from name
namesE   = []
uniquesE  = [] #
campE    = [] # Name of the campaigns the events are in
tierLE    = []

i = 0

# Reads all the Events
for eventfile in glob.glob( prefix+'/dat/events/**/*.lua', recursive=True ):
   print(eventfile)

   with open(eventfile,'r') as f:
      buf = f.read()
   if buf.find('</event>') < 0:
      continue
   p = buf.find('--]]')
   if p < 0:
      continue
   xml = buf[5:p]

   tree = ET.ElementTree(ET.fromstring(xml))
   evt = tree.getroot()

   name = 'Evt: '+evt.attrib['name']
   namesE.append(name)
   namdictE[name] = i

   unique = evt.find('unique')
   if unique == None:
      uniquesE.append(False)
   else:
      uniquesE.append(True)

   # Read the notes
   add_notes( name, evt, "Generic Events", campE, tierLE )

   i += 1

# Generate graph

G=pgv.AGraph(directed=True)

# Create the tier subgraphs
imax = 0
for v in tierL:
   if v != None:
      if v > imax:
         imax = v
for v in tierLE:
   if v != None:
      if v > imax:
         imax = v

if not ignore_tier:
   for i in range( imax+1 ):
      G.add_subgraph(name=str(i)) # TODO: rationalize the use of int and str
      sub = G.get_subgraph(str(i))
      sub.graph_attr['rank']='same'
      sub.add_node(tierNames[i], shape='octagon')
      if i>0:
         G.add_edge(tierNames[i-1], tierNames[i], style='invis')

# Add meta nodes
for node in meta_nodes:
   name = node[0]
   campagn = node[1]
   #if True:
   if campagn == "cluster: Generic Missions" or campagn == "cluster: Generic Events" or ignore_camp:
      G.add_node(name,shape='hexagon',color='red')
   else:
      sub = G.get_subgraph(campagn)
      if sub is None:
         G.add_subgraph(name=campagn,label=campagn)
         sub = G.get_subgraph(campagn)

      sub.add_node(name,shape='hexagon',color='red')


def subgraph_add( name, subN, tier, unique, subN_check, shape ):
   #if True:
   if subN == subN_check or ignore_camp:
      if unique:
         G.add_node(name,shape=shape)
      else:
         G.add_node(name,shape=shape,color='grey')
   else:
      sub = G.get_subgraph(subN)
      if sub is None:
         G.add_subgraph(name=subN,label=subN)
         sub = G.get_subgraph(subN)

      if unique:
         sub.add_node(name,shape=shape)
      else:
         sub.add_node(name,shape=shape,color='grey')

   if tier != None and (not ignore_tier):
      sub = G.get_subgraph(str(tier))
      sub.add_node(name)


# Missions
for i in range(len(names)):
   subgraph_add( names[i], camp[i], tierL[i], uniques[i], "cluster: Generic Missions", 'ellipse' )

# Same thing for events
for i in range(len(namesE)):
   subgraph_add( namesE[i], campE[i], tierLE[i], uniquesE[i], "cluster: Generic Events", 'box' )


for i in range(len(dones)):
   done = dones[i]
   if done == None:
      continue
   name = names[i]
   G.add_edge('Misn: '+done.text,name)

for i in range(len(extra_links)):
   link = extra_links[i]
   G.add_edge( link[0], link[1], label=link[2], color='red' ) #

#G.graph_attr['rank']='same'
G.layout(prog='dot')
#G.layout(prog='neato')
G.draw('missions.svg')



#!/usr/bin/env python3
import os
import sys
import tomllib as toml
import json

HEADER = r"""# Naev toml files
# Copyright (C) YEAR Naev Dev Team
# This file is distributed under the same licence as the naev package.
# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\n"
"Report-Msgid-Bugs-To: https://github.com/naev/naev/issues\n"
"POT-Creation-Date: YEAR-MO-DA HO:MI+ZONE\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: LANGUAGE <LL@li.org>\n"
"Language: \n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=UTF-8\n"
"Content-Transfer-Encoding: 8bit\n"
"""

TRANSLATABLES = {
   "damagetype": {
      "fields": [
         "name",
         "display",
      ],
      "context": "damagetype",

   },
   "slots": {
      'fields': [
         "display",
         "description",
      ],
      "context": "slotproperty",
   },
   "start.toml": {
      'fields': [
         "ship_name",
         "ship_acquired",
         "scenario_name",
      ],
   },
}

DATA = {}
def data( filename, string, context=None ):
   if not context:
      context = "NONE"
   if not DATA.get(context):
      DATA[ context ] = {}
   if not DATA[ context ].get(string):
      DATA[ context ][ string ] = []
   DATA[ context ][ string ].append( filename )

dirnames = sys.argv[2:].copy()
dirnames.sort()

for dirname in dirnames:
   if os.path.isfile(dirname):
      datafile = dirname.split('/')[-1]
      translates = TRANSLATABLES.get(datafile)

      with open( dirname, 'rb' ) as f:
         print(f"processing {dirname}...")
         d = toml.load( f )

         for t in translates["fields"]:
            value = d.get(t)
            if value:
               data( dirname, value, translates.get("context") )

   else:
      for (root, dirs, files) in os.walk( dirname ):
         if not root.endswith('/'):
            root += '/'

         datadir = root.split('/')[-2]
         translates = TRANSLATABLES.get(datadir)
         if not translates:
            print(f"skipping unknown directory {root}")
            continue

         files = list(filter( lambda x: x.endswith(".toml"),files))
         files.sort()

         for filename in files:
            filename = root+filename
            with open( filename, 'rb' ) as f:
               #print(f"processing {filename}...")
               d = toml.load( f )

               for t in translates["fields"]:
                  value = d.get(t)
                  if value:
                     data( filename, value, translates.get("context") )

out = HEADER
for (context,datas) in DATA.items():
   for (value,filenames) in datas.items():
      out += '\n'
      for fn in filenames:
         out += f'#: {fn}:1\n'
      if context != "NONE":
         out += f'msgctxt "{context}"\n'
      out += f'msgid "{value}"\n'
      out += 'msgstr ""\n'

try:
   with open(sys.argv[1],"r") as fin:
      upstream = fin.read()
except:
   upstream = ""

if out != upstream:
   with open(sys.argv[1],"w") as fout:
      fout.write( out )



#!/usr/bin/env python3

import os
import sys
import shutil
import logging

# Configure logging to stderr and stdout
logging.basicConfig(level=logging.INFO, stream=sys.stderr)

def copy_file(source_file, dist_file):
   try:
      os.makedirs(os.path.dirname(dist_file), exist_ok=True)
      shutil.copyfile(source_file, dist_file)
      logging.info(f"File '{source_file}' copied to '{dist_file}'.")
   except Exception as e:
      logging.error(f"Error copying file: {e}")
      sys.exit(1)

if __name__ == "__main__":
   if len(sys.argv) != 3:
      logging.error("Usage: python3 add_to_package.py <INFILE> <OUTFILE>")
      sys.exit(1)

   dist_root = os.getenv("MESON_DIST_ROOT")

   if not dist_root:
      logging.error("Error: MESON_DIST_ROOT environment variable not set.")
      sys.exit(1)

   source_file = sys.argv[1]
   dist_file = os.path.join(dist_root, sys.argv[2])

   copy_file(source_file, dist_file)



#!/usr/bin/env python3
import helper as h
N_ = lambda text: text
data = h.read()

o = data['outfit']
o['@name'] = N_('Corsair Systems')

general = o['general']
del general['shortname']
general['unique'] = None
general['rarity'] = 6
general['$price'] = 1e6
general['description'] = N_("""oh, the ferocious mighty corsair
never sitting in their captain's chair
least they puke out their grub
all over the ship hub
in space is it also mal de mer?""")
del general['slot']['@prop_extra']

specific = o['specific']
del specific['cooldown_time']
specific['ew_detect']['$pri'] += 5
specific['ew_hide'] = -15
specific['lua_inline_post'] = "require('outfits.core_sets.corsair_systems').init()"

data.prisec_only(sec= False)
data.save()



# SPDX-Licence-Identifier: GPL-3.0-or-later

from ctypes import *

from OpenGL.GL import *
from OpenGL.GL.shaders import compileProgram, compileShader
from OpenGL.arrays.vbo import VBO
import glm
import math

from .parse import base_path

vert = '#version 150\n\n' + open(base_path() / 'dat/glsl/material.vert').read()
frag = '#version 150\n\n' + open(base_path() / 'dat/glsl/material_pbr.frag').read()

class RenderObject:
   def __init__(self, program, obj):
      self.mtl_list = obj.mtl_list
      self.vao = glGenVertexArrays(1)

      glBindVertexArray(self.vao)

      vertices = (GLfloat * len(obj.vertices))(*obj.vertices);
      self.vbo = VBO(vertices, GL_STATIC_DRAW, GL_ARRAY_BUFFER)
      self.vbo.bind()

      vertex_attrib = program.attribs["vertex"]
      glEnableVertexAttribArray(vertex_attrib)
      glVertexAttribPointer(vertex_attrib, 3, GL_FLOAT, GL_FALSE, 8 * 4, c_void_p(0))

      vertex_tex_attrib = program.attribs["vertex_tex"]
      glEnableVertexAttribArray(vertex_tex_attrib)
      glVertexAttribPointer(vertex_tex_attrib, 2, GL_FLOAT, GL_FALSE, 8 * 4, c_void_p(3 * 4))

      vertex_normal_attrib = program.attribs["vertex_normal"]
      glEnableVertexAttribArray(vertex_normal_attrib)
      glVertexAttribPointer(vertex_normal_attrib, 2, GL_FLOAT, GL_FALSE, 8 * 4, c_void_p(5 * 4))


class ObjProgram:
   def __init__(self):
      vertex_shader = compileShader(vert, GL_VERTEX_SHADER)
      fragment_shader = compileShader(frag, GL_FRAGMENT_SHADER)
      self.program = compileProgram(vertex_shader, fragment_shader)

      uniform_count = glGetProgramiv(self.program, GL_ACTIVE_UNIFORMS)
      self.uniforms = {glGetActiveUniform(self.program, i)[0].decode() : i for i in range(uniform_count)}

      attrib_count = glGetProgramiv(self.program, GL_ACTIVE_ATTRIBUTES)
      self.attribs = {glGetActiveAttrib(self.program, i)[0].decode() : i for i in range(attrib_count)}

   def use(self):
      glUseProgram(self.program)

   def draw(self, obj, width, height, rot, rot_z):
      glBindVertexArray(obj.vao)

      scale = 5  # matching "camobj" setup in naev-artwork/3D/render.sh

      if width < height:
         scale_w = scale
         scale_h = scale_w * (height / width)
      else:
         scale_h = scale
         scale_w = scale_h * (width / height)

      projection = glm.ortho(-scale_w, scale_w, -scale_h, scale_h, -9*math.sqrt(2), 9*math.sqrt(2))

      model = glm.mat4()
      model = glm.rotate(model, rot + math.pi / 2, glm.vec3(0, 1, 0))
      model = glm.rotate(model, rot_z, glm.vec3(0, 0, 1))

      glUniformMatrix4fv(self.uniforms["projection"], 1, GL_FALSE, projection.to_list())
      glUniformMatrix4fv(self.uniforms["model"], 1, GL_FALSE, model.to_list())

      glEnable(  GL_BLEND )
      glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA )

      glEnable(GL_DEPTH_TEST)
      glDepthFunc(GL_LESS)

      if "map_Kd" in self.uniforms.keys():
         glUniform1i( self.uniforms["map_Kd"], 0 )
      if "map_Ks" in self.uniforms.keys():
         glUniform1i( self.uniforms["map_Ks"], 1 )
      if "map_Ke" in self.uniforms.keys():
         glUniform1i( self.uniforms["map_Ke"], 2 )
      if "map_Bump" in self.uniforms.keys():
         glUniform1i( self.uniforms["map_Bump"], 3 )

      for (mtl, start, count) in obj.mtl_list:
         if "Ns" in self.uniforms.keys():
            glUniform1f(self.uniforms["Ns"], mtl.Ns)
         if "Kd" in self.uniforms.keys():
            glUniform3f(self.uniforms["Kd"], *mtl.Kd)
         if "Ka" in self.uniforms.keys():
            glUniform3f(self.uniforms["Ka"], *mtl.Ka)
         if "Ks" in self.uniforms.keys():
            glUniform3f(self.uniforms["Ks"], *mtl.Ks)
         if "Ke" in self.uniforms.keys():
            glUniform3f(self.uniforms["Ke"], *mtl.Ke)
         if "Ni" in self.uniforms.keys():
            glUniform1f(self.uniforms["Ni"], mtl.Ni)
         if "d" in self.uniforms.keys():
            glUniform1f(self.uniforms["d"], mtl.d)
         if "bm" in self.uniforms.keys():
            glUniform1f(self.uniforms["bm"], mtl.bm)

         if "map_Kd" in self.uniforms.keys():
            glActiveTexture(GL_TEXTURE0)
            glBindTexture(GL_TEXTURE_2D, mtl.map_Kd)

         if "map_Ks" in self.uniforms.keys():
            glActiveTexture(GL_TEXTURE1)
            glBindTexture(GL_TEXTURE_2D, mtl.map_Ks)

         if "map_Ke" in self.uniforms.keys():
            glActiveTexture(GL_TEXTURE2)
            glBindTexture(GL_TEXTURE_2D, mtl.map_Ke)

         if "map_Bump" in self.uniforms.keys():
            glActiveTexture(GL_TEXTURE3)
            glBindTexture(GL_TEXTURE_2D, mtl.map_Bump)

         glDrawArrays(GL_TRIANGLES, start, count)

      self.gl_checkErr()

   def gl_checkErr(self):
      err = glGetError()
      if err != 0:
         print(gluErrorString(err))



class UnionFind:
   '''Represents a partition of range(n) into mergeable disjoint subsets.'''

   def __init__(self, n):
      self.parent = list(range(n))
      self.rank = [0] * n

   def union(self, x, y):
      '''Declares x and y to be in the same subset.'''
      x = self.find(x)
      y = self.find(y)
      if self.rank[x] > self.rank[y]:
         self.parent[y] = x
      else:
         self.parent[x] = y
         if self.rank[x] == self.rank[y]:
            self.rank[y] += 1

   def find(self, x):
      '''Finds the designated representative of the subset containing x.'''
      if self.parent[x] != x:
         self.parent[x] = self.find(self.parent[x])
      return self.parent[x]

   def findall(self):
      '''Returns a designated representative of each subset.'''
      return [i for i,x in enumerate(self.parent) if i==x]
